// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader;

import jason.service.pool.*;
import java.io.*;
import jason.service.crusader.datastructure.graph.*;


/**This is the main frame of the Crusader Servlet Engine. This class is responsible to co-ordinate all
*the Servlet related activity. This is the only point of interaction between the webserver and the Servlet
*Engine. This class is responsible to execute each Servlet in it's own thread. This is also the point from
*which the administrator can set Security policies and other administrative activities.<br/>
*
*Author - Sanjay"Ironluca"Debnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 04/05/2001
*/
public class Crusader{

    /**Contains the default thread pool size - 50.*/
    protected static int DEFAULT_POOL_SIZE=50;

    /**This contains the thread pool for the CrusaderCore.*/
    protected ThreadPoolInt crusaderCorePool;

    /**Contains the graph of data related to the ServletContext information - one node per context.*/
    // Modified by AB fpr avoiding a NullPointerException
    protected static GraphNode contextInfoRoot=new GraphNode("/");


    /**
     *Default constructor calls the overloaded constructor with default pool size (50). */
    public Crusader(){
        this( DEFAULT_POOL_SIZE );
    }

    /**
     *Overloaded constructor .
     *@param <b>threadPoolSize</b> The target size of the thread pool. */
    public Crusader(int threadPoolSize){

	try{
	    crusaderCorePool=new ThreadPool( jason.service.crusader.CrusaderCore.class, threadPoolSize );
	} catch( Exception exc ) {
	    exc.printStackTrace();
	}

	contextInfoRoot= new GraphNode("/");
    }

    /**Returns the root of the graph that contains the ServletContext information.*/
    public static GraphNode getContextInfoRoot(){ 
	return contextInfoRoot; 
    }

    /**
     *This is the interface method which shall initiate the Servlet execution - the main server
     *process should only use this method to initiate Servlet execution.<br/>
     *@param <b>clientIStream</b> The java.io.InputStream from the client.
     *@param <b>clientOStream</b> The java.io.OutputStream to the client. */
    public synchronized void executeServlet(InputStream clientIStream, OutputStream clientOStream){

	//Retrieve a thread capsule from the pool
	ThreadCapsuleInt threadCapsule = crusaderCorePool.getThreadCapsule();

	if( threadCapsule == null ){
	    System.out.println("Null ThreadCapsule - Pool locked");
	    return;
	}
	else{
	    CrusaderCore crusaderCore=( CrusaderCore )( threadCapsule.getCoreThread() );
	    crusaderCore.setClientOStream( clientOStream );
	    threadCapsule.activateThreadCapsule();
	}
    }

}

