// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader;

import jason.core.*;
import jason.core.pool.SocketTask;
import jason.core.pool.Task;

import jason.service.crusader.javaximpl.*;

import jason.service.http1_0.*;
import jason.service.http1_0.framework.*;

import jason.framework.protocol.*;
import jason.framework.protocol.http.*;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * <p>
 * I have added a relation with the http1.0 framework for building a 
 * HTTP1.0 response. It should be removed later.
 * </p>
 *
 * @author "Alexandre Brillant" <djefer@club-internet.fr>
 * @version 1.1
 * @since 1.0
 * @see AbstractService */
public class CrusaderService extends jason.core.HTTPService {

    public static String TYPE = "servlet";

    public CrusaderService(){
	super();
	try {
	    setType( new ServiceType( ServiceType.JASON_SERVICE, new String[] { TYPE, "crusader" } ) );
	} catch( ServiceException exc ) {
	    fireCriticalErrorDetected( exc );
	}
    }

    // Try to reload the configation for each start 
    public void start() {
	initServlets();
	super.start();
    }

    // Store the servlet
    // This is a mapping between an URI and a servlet class
    private void initServlets() {
	String servlets = getStringProperty( "servlet" );
	StringTokenizer st = new StringTokenizer( servlets, " ,;", false );
	servletClass = new Properties();
	while ( st.hasMoreTokens() ) {
	    String line = st.nextToken();
	    int i = line.indexOf( "=" );
	    if ( i > 0 ) {
		String uri = line.substring( 0, i );
		String _class = line.substring( i + 1, line.length() );
		servletClass.put( uri, _class );
		sendInformationMessage( "Mapping " + uri + " --> " + _class );
	    }
	}
	String classes = getStringProperty( "classes" );
	CrusaderUtil.setClassDir( classes );
    }

    protected Task getTaskForSocket( Socket host ) throws ServiceException {
	return new ServletProcessing( host );
    }

    /**
     * @param out OutputStream towards the Browser
     * @param htmlMessage HTML Content
     * @param complete Decide to wrappe the message with a html body
     * @exception IOException if an error occurs  */
    private void sendHTMLMessage( OutputStream out, String htmlMessage, boolean complete ) throws IOException {
	if ( "".equals( htmlMessage ) )
	    htmlMessage = "JASon Default Response...";
	StringBuffer sb = new StringBuffer( complete ? "<html><body>" : "" ).append( htmlMessage ).append( complete ? "</body></html>" : "" );
	DocHttp1Factory factory = new DocHttp1Factory( new Hashtable() { public Object get( Object key ) { return "text/html"; } } );
	IntDocHttp http1 = factory.buildDoc( "JASon", "/jason.service.message", IntHttpCode.CO_OK, sb );
	out.write( http1.getDocument().toString().getBytes() );
    }

    private Properties servletClass;

    /////////////////////////

    class ServletProcessing extends SocketTask {

	private InputStream in;
	private OutputStream out;

	public ServletProcessing( Socket host ) throws ServiceException {
	    super( host );
	    try {
		in = socket.getInputStream();
		out = socket.getOutputStream();
	    } catch( IOException exc ) {
		throw new ServiceException( exc.getMessage() );
	    }
	}

	public void run() {
	    try {
		try {
		    HttpRequestParser parser = ProtocolFactory.getHttpRequestParser();
		    parser.parse( in );
		    String uri = parser.getRequestURI();

		    if ( !servletClass.containsKey( uri ) ) 
			sendHTMLMessage( out, "Unknown Servlet for this URI :" + uri, false );
		    else {
			// Servlet working
			CrusaderCore crusadercore = new CrusaderCore( parser );
			String servlet = servletClass.getProperty( uri );
			crusadercore.setClassName( servlet );
			crusadercore.setClientOStream( out );

			try {
			    crusadercore.run();
			} catch( Exception exc ) {
			    sendErrorMessage( "Can't initialize " + servlet );
			    sendHTMLMessage( out, "Can't initialize <br>Please check the classpath" + servlet, true );
			    sendInnerError( exc );
			}
		    }

		} catch( Exception exc ) {
		    sendErrorMessage( "Error :" + exc.getMessage() );
		}
	    } finally {
		try {
		    in.close();
		    out.close();
		    socket.close();
		} catch( Exception exc ) {}
	    }
	}
    }

}
