// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


/**This class encapsulates a servlet and other related information regarding the servlet. Objects of this
*class shall be used to encapsulate a servlet & other relevant information regarding the Servlet
*by the engine.<br/>
*
*Author - Sanjey"Ironluca"Debnath <a href="Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 15/04/01
*
*/

package jason.service.crusader;

import javax.servlet.*;
import java.io.*;

public class ServletCapsule implements ServletCapsuleInt, Servlet{

/*This variable is only for testing - remove after testing*/
public static int capsuleID=0;
public static int servletObjectID=0;
/*This method is only for testing - remove after testing*/
public int getCapsuleID(){return capsuleID;}
public int getServletObjectID(){return servletObjectID;}
/*Remove the constructor after testing.*/
public ServletCapsule(){
/*
this.servletName=servletName;
this.fileName=fileName;
*/
capsuleID++;

}//constructor closing





/**Denotes the time the servlet was last modified.*/
        protected long lastModified;

/**The fully qualified name of the Servlet class.*/
        protected String servletName;

/**Contains the filename of the servlet.*/
        protected String fileName;

/*The servlet object - encapsulated.*/
        protected Servlet servlet;

/**Indicates wether the servlet implements javax.servlet.SingleThreadModel or not.*/
        protected boolean singleThreadModel;

/**Indicates wether the servlet encapsulated is ready for servicing clients' requests or not.
It shall be exclusively used for servlets with SingleThreadModel and to prevent the Servlet
instance to handle any more requests prio to decapsulation (if the requirement is felt).<br/>
<b>NOTE: SERVLETS IMPLEMENTING SingleThreadModel SHOULD NOT BE
DECAPSULATED, THEY SHOULD BE ABANDONED ALTOGETHER. IF A CLIENT IS UNDER
PROCESSING - THE CLIENT HAS A HIGH CHANCE OF GETTING LOST.</b>*/
        protected boolean readyForService;

/**
*Returns the lastModified time of the .class file of the servlet. It shall be used for Servlet reloading
*/
        public long getLastModified(){return lastModified;}

/**
*This should not be called until and unless the Servlet has been reloaded.
@param <b>lastModified</b> The <b>long</b> value which indicates the time the Servlet .class file was last modified.
*/
        public void setLastModified(long lastModified){this.lastModified=lastModified;}


/**
*Returns the fully qualified class name of the Servlet that is stoted in the capsule.
@return java.lang.String Indicating the fully qualified class name of the Servlet file.
*/
        public String getServletName(){return servletName;}


/**
*The String supplied shall have to be the fully qualified class name of the Servlet.
@param <b>servletName</b> The fully qualified class name of the Servlet.
*/
        public void setServletName(String servletName){this.servletName=servletName;}


/**
*Returns the real & absolute path name of the .class file of the Servlet
*/
        public String getFileName(){return fileName;}


/**
*The file name supplied shall have to be the absolute real path name of the .class file of the servlet - URLs 
*are strictly not allowed. The name have to be of the form native to the platform. This method shall not
*perform any conversion.
*/
        public void setFileName(String fileName){this.fileName=fileName;}


/**
*Returns the Servlet instance encapsulated
*/
        public Servlet getServlet(){return servlet;}


/**
*This method replaces the Servlet encapsulated in the capsule. Extreme caution must be exercised
*while decapsulating a servlet - for normal Servlets, the readyForService variable should be set to "false"
*and the decapsulation should be conducted. For Servlets which implement SingleThreadModel, decapsulation
*should not be attempted.
*/
        public void setServlet(Servlet servlet){this.servlet=servlet;servletObjectID++;/*Remove this line after testing*/}


/**
*Returns wether a Servlet encapsulated implements SingleThreadModel or not
*/
        public boolean isSingleThreadModel(){return singleThreadModel;}


/**
*Sets the singleThreadModel status for a Servlet that is encapsulated
*/
        public void setSingleThreadModel(boolean singleThreadModel){this.singleThreadModel=singleThreadModel;}


/**
*Returns wether a Servlet encapsulated is ready to service a client or not - it shall be predominantly
*used with Servlets implementing SingleThreadModel and during Servlet decapsulation.
*/
        public boolean isReadyForService(){return readyForService;}


/**
*Sets the readyForService status for an encapsulated Servlet
*/
        public void setReadyForService(boolean readyForService){this.readyForService=readyForService;}


/*
*Overridden methods from Servlet interface calls equivalent methods on the underlying Servlet
*that is encapsulated.
*/
        public void init(ServletConfig config) throws ServletException{servlet.init(config);}
        public void service(ServletRequest request, ServletResponse response)throws ServletException, IOException{servlet.service(request, response);}
        public void destroy(){servlet.destroy();}
        public ServletConfig getServletConfig(){return servlet.getServletConfig();}
        public String getServletInfo(){return servlet.getServletInfo();}

}
