// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader.datastructure.graph;

import java.util.*;

/**
*This class is an limited implementation of a graph datastructure. This class represents a node in a data graph.<br/>
*
*Author - Sanjay"Ironluca"Debnath <a href="Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 2/07/2001<br/>
*
*/

public class GraphNode{

/**The refference to the prent GraphNode of this node.*/
GraphNode parent;

/**The name of a node - unique in it's context.*/
protected String nodeName;

/**Stores the ttributes of the node - mostly to be set by the server or not at all.*/
protected Hashtable nodeAttributes;

/**The parameters associated with a node.*/
protected Hashtable nodeProperties;

/**This variable stores the vertices or the traversal path from this node to child nodes if any.*/
protected Hashtable nodeList;

/**No args default constructor.*/
public GraphNode(){}

/**Parameterised constructor - initializes the name of this node to the name passed.
@param nodeName The name to which this node is to be set.*/
public GraphNode(String nodeName){

        this.nodeName=nodeName;

}//parameterised constructor closing


/**Returns the unique name of the node in this context - <b>NOTE</b> - it does not return the fully qualified
node name from the root.
@return java.lang.String The unique name of this node in the context, or "null" if one does not exist.*/
public String getNodeName(){

        if(nodeName == null)return null;
        return nodeName;

}//getNodeName closing

/**Changes the name of the node.
@param nodeName The name to which the node has to be set.*/
public void setNodeName(String nodeName){this.nodeName=nodeName;}

/**Retrieves a property of a node, if present as indicated by the property name or "null" if the named 
property does not exist.
@param name The name of the property to be retrieved.
@return java.lang.String The value of the parameter as a String or "null" if one does not exist.*/
public String getNodeProperty(String name){

        if(nodeProperties == null) return null;
        else return (String)(nodeProperties.get(name));

}//getProperty closing

/**This method is responsible to set a particular property of the node.
@param name The name of the parameter to be set.
@param value The value of the parameter.
@return java.lang.String The old value of the parameter if any - or "null".*/
public String setNodeProperty(String name, String value){

        if(nodeProperties == null) nodeProperties=new Hashtable();
        return (String) (nodeProperties.put(name,value));

}//setProperty closing

public Enumeration getNodePropertyNames(){return nodeProperties.keys();}



/**This method returns an named attribute of the node or "null" if one does not exist.
@param name The name of the attribute the node carries.
@return java.lang.String The value of the attribute if it exists or "null".*/
public Object getNodeAttribute(String name){

        if(nodeAttributes == null)return null;
        else return nodeAttributes.get(name);

}//getNodeAttribute closing

/**This method is responsible to set a particular attribute of the node.
@param name The name of the attribute to be set.
@param value The value of the attribute.
@return java.lang.String The old value of the attribute if any - or "null".*/
public String setNodeAttribute(String name, Object value){

        if(nodeAttributes == null) nodeAttributes=new Hashtable();
        return (String) (nodeAttributes.put(name,value));

}//setNodeAttribute closing

/**Returns the names of the attributes of the node.
@return java.util.Enumeration The names of all the nodeAttributes as String.*/
public Enumeration getNodeAttributeNames(){return nodeAttributes.keys();}

/**Removes one particular attribute as indicated by the name
@param attributeName The name of the attribute to be removed.*/
public void removeNodeAttribute(String attributeName){nodeAttributes.remove(attributeName);}

/**Returns the particular named child of this node if it exists or "null" if it does not.
@param name The name of the child node required.
@return jason.service.crusader.datastructure.graph.GraphNode The child node if it exists or "null" if it does not.*/
public GraphNode getChildNode(String name){

        if(nodeList == null)return null;
        else return (GraphNode)(nodeList.get(name));

}//getChildNode closing

/**Sets a new child node to this node. If there exists a child with a specific name, it shall be replaced.
@param name The name under which the child is to be stored.
@param childNode The child node that is to be stored.
@return The old child if one exists or "null" if there is none.*/
public GraphNode setChildNode(String name, GraphNode childNode){

        if(nodeList == null)nodeList=new Hashtable();
        return (GraphNode)(nodeList.put(name, childNode));

}//setChildNode closing

/**This method retrieves the parent of this node - if any or null.
@return jason.service.crusader.datastructure.GraphNode The parent node or "null".*/
public GraphNode getParent(){return parent;}

/**This method sets a new parent to this node.
@param parent The new parent.
@return jason.service.crusader.datastructure.graph.GraphNode The old parent if any or "null".*/
public GraphNode setParent(GraphNode parent){

        GraphNode tempParent=this.parent;
        this.parent=parent;
        return tempParent;

}//setParent


/**Convenient method to check wether this node has a parent or not. Usually all the nodes will have parent
except the root node.
@return boolean If true, this node has a parent and is not the root node. If false, most likely that this node is 
the root node.*/
public boolean hasParent(){

        if(parent == null) return false;
        else return true;

}//hasParent closing


/**Convenience method to check wether this node has children or not.*/
public boolean hasChildNodes(){

        if(nodeList == null)return false;
        else return true;

}//hasChildNodes closing


/**This is a convenience method to add a nodeProperty to one particular node in the tree.
@param childNodeName The name of the child node to the existing evaluation node to which this nodeProperty
is to be added.
@param propertyName The name of the property.
@param propertyValue The value of the property.*/ 
public void addNodePropertyToTree(String childNodeName, String propertyName, String propertyValue){

        this.getChildFromTree(childNodeName,true).setNodeProperty(propertyName, propertyValue);

}//addNodePropertyToTree closing


/**This is a convenience method to add a nodeAttribute to one particular node in the tree.
@param childNodeName The name of the child node to the existing evaluation node to which this nodeProperty
is to be added.
@param attributeName The name of the attribute.
@param attributeValue The value of the attribute.*/
public void addNodeAttributeToTree(String childNodeName, String attributeName, String attributeValue){

        this.getChildFromTree(childNodeName,true).setNodeAttribute(attributeName, attributeValue);

}//addAttributeToTree closing


/**This method retrieves a particular child node from the tree. The node evaluation is done with respect to
the current node i.e. the node on which this method is called.
@param childNodeName The name of the child node with respect to this node.
@param create Creates the series of child nodes and appends them to the tree if "true" - otherwise not.
@return jason.service.crusader.datastructure.graph.GraphNode The child node - if one exists or else "null"*/
public GraphNode getChildFromTree(String childNodeName, boolean create){

        String tempName=null;
        GraphNode tempChild=null;
        GraphNode tempThis=this;//The present node of evaluation

        StringTokenizer tokenizer=new StringTokenizer(childNodeName,"/");
        while(tokenizer.hasMoreTokens()){

                tempName=tokenizer.nextToken();
                tempChild=tempThis.getChildNode(tempName);

                if(tempChild == null && create){

                        tempChild=new GraphNode(tempName);
                        tempChild.setParent(tempThis);

                        tempThis.setChildNode(tempName,tempChild);
                        tempThis=tempChild;

                        while(tokenizer.hasMoreTokens()){

                                tempName=tokenizer.nextToken();
                                tempChild=new GraphNode(tempName);
                                tempChild.setParent(tempThis);

                                tempThis.setChildNode(tempName,tempChild);
                                tempThis=tempChild;

                        }//while closing

                        break;

                }//if closing
               if(tempChild == null && !create)break;
                tempThis=tempChild;

        }//while closing

        return tempThis;

}//getChildFromTree closing


}
