// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader.javaximpl;

import java.io.*;
import java.util.*;
import java.text.*;
import java.security.*;
import javax.servlet.http.*;

import jason.framework.protocol.http.*;

/**
* This class represents the concrete implementation of the javax.servlet.http.HttpServletRequest
* This class extends the {@link jason.service.crusader.ServletRequestImpl ServletRequestImpl} and derives it's base functionalities. This class also implements
* javax.servlet.http.HttpServletRequest.
*
* @author anjay"Ironluca"Debnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a>
* @author Alexandre Brillant <abrillant@wanadoo.fr>
* @version 1.1
*/
public class HttpServletRequestImpl extends ServletRequestImpl implements HttpServletRequest{

    public HttpServletRequestImpl( HttpRequestParser parser ){
	super( parser );
    }

    public String getAuthType(){return null;}
    public String getContextPath(){return null;}
    public Cookie [] getCookies(){return null;}

    public long getDateHeader( String headerName ){
        String stringDate = parser.getHeader( headerName );
        if ( stringDate == null ) return -1;
        try{
	    return Long.parseLong( stringDate );
        }catch(NumberFormatException nfe){}

        try{
	    DateFormat dateFormat = DateFormat.getDateInstance();
	    Date d = dateFormat.parse(stringDate);
	    return d.getTime();
        } catch(ParseException pe){
	    throw new IllegalArgumentException("Illegal Date" );
	}
    }

    public String getHeader( String headerName ) {
        return parser.getHeader( headerName );
    }

    public Enumeration getHeaderNames() {
        return parser.getHeaderNames();
    }

    public Enumeration getHeaders(String headerName){
	return null;
    }

    public int getIntHeader(String headerName){
        String value = getHeader( headerName );
        if( value == null ) return -1;
        else return Integer.parseInt( value );
    }

    public String getMethod(){
        return parser.getMethod();
    }

    public String getPathInfo() { return null; }
    public String getPathTranslated() { return null; }

    public String getQueryString(){
	return parser.getRequestURIQuery();
    }

    public String getRemoteUser() { return null; }
    public String getRequestedSessionId() { return null; }


    public String getRequestURI(){
	return parser.getRequestURI();
    }

    public String getServletPath(){ return null; }
    public HttpSession getSession(){ return null; }
    public HttpSession getSession(boolean create) { return null; }
    public Principal getUserPrincipal() {return null; }
    public boolean isRequestedSessionIdFromCookie() { return false; }
    public boolean isRequestedSessionIdFromUrl() { return false; }
    public boolean isRequestedSessionIdFromURL() { return false; }
    public boolean isRequestedSessionIdValid() { return false; }
    public boolean isUserInRole(String role) { return false; }

}
