// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader.javaximpl;

import javax.servlet.*;
import java.util.*;

import jason.service.crusader.*;

/**This class is the concrete implementation of java.servlet.ServletConfig. The config object is responsible to give
* a servlet access to the configuration parameters (if any) specific to a servlet. The configuration parameters are
* a subset of all the configuration parameters stored by the servlet engine for all the contexts. The ServletConfig
*interface may be considered to be a Servlet's view into the configuration data as maintained by the servlet engine.
*Therefore, there is only one ServletConfig object per servlet.<br/>
*
*Author - Sanjay"Ironluca"Debnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 29/04/2001
*
*
*/

public class ServletConfigImpl implements ServletConfig{

    /**Contains the refference to the Servlet/ServletCapsule for which it is acting as the Config*/
    protected ServletCapsuleInt servletCapsule;

    /**Contains the Context associated with this Servlet/ServletCapsule.*/
    protected ServletContext servletContext;


    public String getInitParameter(String parameterName){ return servletContext.getInitParameter(parameterName);}
    public Enumeration getInitParameterNames(){return servletContext.getInitParameterNames();}


    /**
     *The present implementation of this method returns a new Context object if the "servletContext" variable
     *is null. Otherwise it returns the servletContext. The "servletContext" is to be set by CrusaderCore in the run() method.
     */
    public ServletContext getServletContext(){

	if(servletContext == null) return new ServletContextImpl();
	else return servletContext;

    }//getServletContext closing

    /**
     *This method returns the Servlet name - in effect the fully qualified class name
     */
    public String getServletName(){ 
	if (servletCapsule != null ) return servletCapsule.getServletName();
	else return "?";
    }


    /**This methods is for internal use only.*/
    public ServletCapsuleInt getServletCapsule(){return servletCapsule;}

    /**This methods is for internal use only.*/
    public void setServletCapsule(ServletCapsuleInt servletCapsule){this.servletCapsule=servletCapsule;}

    /**This methods is for internal use only.*/
    public void setServletContext(ServletContext servletContext){this.servletContext=servletContext;}

}//class closing
