// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader.javaximpl;

import javax.servlet.*;
import java.util.*;
import java.io.*;
import java.net.*;

import jason.service.crusader.*;
import jason.service.crusader.datastructure.graph.*;

/**This class is a concrete implementation of javax.servlet.ServletContext.
*It contains a {@link jason.service.crusader.datastructure.graph.GraphNode GraphNode} object - that serve as the container
*for initParameters and attributes.<br/>
*
*Author - Sanjay"Ironluca"Dabnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 29/04/2001
*
*/

public class ServletContextImpl implements ServletContext{

/**Contains the name under which this ServletContext is bound in the data graph tree.
*/
        protected String contextName;

/**Contaions the refference to the {@link jason.service.crusader.datastructure.graph.GraphNode GraphNode}
*object which in turn contains the ServletContext information for a particular context.
*/
        protected GraphNode contextInfo;

/*
        protected Hashtable initParameterHash;//Contains the initParameters
        protected Hashtable attributeHash;//Contains the attributes

        public Hashtable getInitParameterHash(){return initParameterHash;}
        public Hashtable getAttributeHash(){return attributeHash;}
*/

/**No args default constructor - does nothing*/
        public ServletContextImpl(){}

/**The parameterised constructor retrieves a context with the given name. If the context does not exist, one is
created.
@param contextName The name of the context to retrieve.*/
        public ServletContextImpl(String contextName){
/*
                initParameterHash=new Hashtable();
                attributeHash=new Hashtable();
*/
                this.contextName=contextName;
                contextInfo=Crusader.getContextInfoRoot().getChildFromTree(contextName,true);

        }//default constructor closing

/**Returns the context name to which this ServletContext object is bound.
@return java.lang.String The name under which this context object is bound.
*/
        public String getContextName(){return contextName;}

/**This method sets the context name for this ServletContext. If a different context name is set - the internal
{@link jason.service.crusader.datastructure.graph.GraphNode GraphNode} is changed to point to the new node in the
data graph tree.*/
        public void setContextName(String contextName){

                this.contextName=contextName;
                contextInfo=Crusader.getContextInfoRoot().getChildFromTree(contextName,true);

}//setContextName closing


/*Methods for javax.servlet.ServletContext interface*/

        public Object getAttribute(String attributeName){

                return contextInfo.getNodeAttribute(attributeName);

        }//getAttribute closing

        public Enumeration getAttributeNames(){

                return contextInfo.getNodeAttributeNames();

        }//getAttributeNames closing

        public ServletContext getContext(String URLName){return null;}
        public int getMajorVersion(){return 0;}
        public String getMimeType(String fileName){return null;}
        public int getMinorVersion(){return 0;}
        public String getRealPath(String vertualPath){return null;}
        public RequestDispatcher getRequestDispatcher(String URLPath){return null;}
        public URL getResource(String path){return null;}
        public InputStream getResourceAsStream(String path){return null;}
        public String getServletInfo(){return null;}
        public void log(String message){};
        public void log(String message, Throwable throwable){}

        public void removeAttribute(String attributeName){

                contextInfo.removeNodeAttribute(attributeName);

        }//removeAttribute closing

        public void setAttribute(String attributeName, Object attributeValue){

                contextInfo.setNodeAttribute(attributeName,attributeValue);

        }//setAttribute closing


/*These set of methods have been deprecated - it only exists for backward compatibility.*/
/*All the methods shall always return null.*/

        public Servlet getServlet(String servletName){return null;}
        public Enumeration getServlets(){return null;}
        public Enumeration getServletNames(){return null;}
        public void log(Exception exception, String message){}


/*These methods have been added in the later release (2.2)*/

        public String getServerInfo(){return null;}
        public RequestDispatcher getNamedDispatcher(String dispatcherName){return null;}

        public Enumeration getInitParameterNames(){

                return contextInfo.getNodePropertyNames();

        }//getInitParameterNames closing

        public String getInitParameter(String initParameterName){

                return contextInfo.getNodeProperty(initParameterName);

        }//getInitParameter closing


}
