// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.crusader.javaximpl;

import javax.servlet.*;
import java.io.*;
import java.util.*;

import jason.framework.protocol.http.*;

/*
 * This class provides a concrete implementstion of javax.servlet.ServletRequest and shall be
* extended by the concrete implementation of javax.servlet.http.HttpServletRequest.
*
* @author Sanjay"Ironluca"Debanth<Ironluca@yahoo.com>
* @author Alexandre Brillant <abrillant@wanadoo.fr>
* @version 1.1 : Support for the new framework
*/

public class ServletRequestImpl implements ServletRequest{
    protected ServletInputStream toClientStream = null;
    protected BufferedReader reader = null;
    protected HttpRequestParser parser = null;

    public ServletRequestImpl( HttpRequestParser parser ) {
	this.parser = parser;

	byte[] content = parser.getMessageBody();
	if ( content == null )
	    content = new byte[] {};

	ByteArrayInputStream buffer = new ByteArrayInputStream( content );
	toClientStream = new ServletInputStreamImpl( buffer );
	reader = new BufferedReader( new InputStreamReader( buffer ) );
    }

    public Object getAttribute( String attributreName ) { return null; }
    public Enumeration getAttributeNames() { return null; }
    public String getCharacterEncoding() { return null; }

    /*
     *Returns the length of the message (in bytes) the client has sent - essentially returns the length of the
     *length of the messageBody byte array in he HttpHandler class. If the messageBody is null, this method
     *returns "0" */
    public int getContentLength(){
	String length = parser.getHeader( "content-length" );
	if ( length == null ) return 0;
	else
	    try {
		return Integer.parseInt( length);
	    } catch( NumberFormatException exc ) { 
		return 0;
	    }
    }

    /*
     *Returns the value of the "Content-Type" header if the client has sent one or else "null" */
    public String getContentType(){
	return parser.getHeader( "content-type" );
    }

    /*
     *Returns the ServletInputStream to the servlet */
    public ServletInputStream getInputStream(){
	return toClientStream;
    }

    /*
     *Returns the value of the parameter or null if not sent. If the parameter contains more than one value
     *only one value out of those shall be returned. */
    public String getParameter(String parameterName){
	return parser.getParameter( parameterName );
    }

    /*
     *Returns an Enumeration (EnumerationImpl) of names of parameters */
    public Enumeration getParameterNames(){
	return parser.getParameterNames();
    }

    // To be corrected later
    public String[] getParameterValues( String parameterName ){
	String value = getParameter( parameterName );
	if ( value == null ) 
	    return null;
	return new String[] { value };
    }

    /*
     *Returns the protocol along with the version (HTTP/1.1) etc - this will be modified later to only return the protocol */
    public String getProtocol() {
	return parser.getProtocol();
    }

    /*
     *Returns the BufferedReader connected to the ByteArrayOutputStream */
    public BufferedReader getReader(){
	return reader;
    }

    public String getRealPath(String vertualPath){
	return null;
    }

    public String getRemoteAddr(){
	return null;
    }

    public String getRemoteHost(){
	return null;
    }

    public String getScheme(){
	return null;
    }

    public String getServerName(){
	return null;
    }

    public int getServerPort(){
	return 0;
    }

    public void setAttribute(String attributeName, Object attributeValue) {}

    /*
     * @returns the default locale being used */
    public Locale getLocale(){
	return Locale.getDefault();
    }

    public Enumeration getLocales(){ return null; }
    public RequestDispatcher getRequestDispatcher(String name){ return null; }
    public boolean isSecure(){ return false; }
    public Map getParameterMap(){ return null; }
    public void removeAttribute( String attributeName ) {}
    public void setCharacterEncoding( String characterEncoding ) {}

}
