// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


/*This class is the concrete implementation of javax.servlet.ServletResponse interface.
*This class encapsulates the common functionalities of a ServletResponse and shall be extended
*by concrete implementation of javax.servlet.http.HttpServletResponce interface.
*
*Author - Sanjay"Ironluca"Debnath<Ironluca@yahoo.com>
*Last Modified - 05/05/2001
*
*/

package jason.service.crusader.javaximpl;

import javax.servlet.*;
import java.io.*;
import java.util.*;


public class ServletResponseImpl implements ServletResponse{

/** TAKE PRECAUTIONS IF Servlet STARTS SENDING TERABYTES OF INFORMATION
KEEP A OUTPUTSTREAM TO THE CLIENT AND SEND EVERYTHING OUT THAT WAY
THIS IS A POTENTIAL BUG - SHOULD BE RECTIFIED**/

        protected Hashtable headerHash;//This Hashtable is responsible to store all the headers that is set
                                                               //by the Servlet to be passed on to the client.

        protected byte [] messageBody;//It shall store the dynamic content generated by the Servlet and shall be passed
                                                            //to the client - NO MODIFICATIONS SHALL BE MADE TO THIS DATA.

        protected ByteArrayOutputStream messageBodyByteStream;//This stream shall be encapsulated in a
                                                                                                              //ServletOutputStream and presented to the Servlet

        protected ServletOutputStream outGatewayStream;//This shall wrap a ByteArrayOutputStream and shall
                                                                                             //be returned by the getOutputStream() call.

        protected PrintWriter writer;//The PrintWriter is implemented over the ByteArrayOutputStream


        public ServletResponseImpl(){

                headerHash=new Hashtable();
                messageBodyByteStream=new ByteArrayOutputStream();
                outGatewayStream=new ServletOutputStreamImpl(messageBodyByteStream);
                writer=new PrintWriter(messageBodyByteStream,true);

        }//default constructor closing


/*
*The call to this method reinitializes the ServletResponseImpl object for reuse. This is only called
*by CrusaderCore - for internal use.
*/
        public void reinitializeServletResponse(){

                headerHash.clear();//Clears the headerHash
                headerHash.putAll(DefaultResponseHeaderHash.getDefaultResponseHeader());//Puts the default values

                messageBodyByteStream.reset();//Resets the ByteArrayOutputStream

        }//reinitializeServletResponse closing


/*
*Returns the Hashtable storing the headers set by the Servlet - for internal use
*/
        public Hashtable getHeaderHash(){

                return headerHash;

        }//getHeaderHash closing


/*
*Returns the message body sent by the Servlet
*/
        public byte [] getMessageBody(){

                messageBody=messageBodyByteStream.toByteArray();
                return messageBody;

        }//getMessageBody



        public String getCharacterEncoding(){return null;}
        public ServletOutputStream getOutputStream(){

                //Returns the ServletOutputStream encapsulated in this Response object.
                return outGatewayStream;

        }//getOutputStream closing

/*
*Returns the Writer object from this Response.
*/
        public PrintWriter getWriter(){

                return writer;

        }//getWriter closing


/*
*Sets the Content-Type key in the headerHash to the required value
*/
        public void setContentLength(int contentLength){

                headerHash.put("Content-Length",String.valueOf(contentLength));

        }//setContentLength closing


/*
*Sets the Content-Type key in the headerHash to the required value.
*/
        public void setContentType(String contentType){

                headerHash.put("Content-Type",contentType);

        }//setContentType closing

/*These methods are added by the Servlet v2.2*/

/*
*Causes the underlying ByteArrayOutputStream to discard the accumulated bytes
*/
        public void reset(){

                messageBodyByteStream.reset();

        }//reset closing

        public void setLocale(Locale locale){/*Not implemented as yet*/}

/*
*Returns the default Locale being used.
*/
        public Locale getLocale(){

                return Locale.getDefault();

        }//getLocale closing

        public void setBufferSize(int bufferSize){/*this method shall not be implemented, as is, it takes no action*/}
        public void flushBuffer(){}

/*
*Returns the buffer size of the underlying ByteArrayOutputStream
*/
        public int getBufferSize(){

                return messageBodyByteStream.size();

        }//getBuffer closing

        public boolean isCommitted(){return false;}

}
