// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.http1_0;

import java.util.*;

/**
 * Header from the RFC 677
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 */
class Attribute {

    public Attribute( String type, String value ) {
	this.type = type;
	this.value = value;
    }

    public Attribute( String val ) {
	int index = val.indexOf( sep );
	if ( index < 0 ) {
	    type = val;
	    value = "";
	} else {
	    type = val.substring( 0, index );
	    value = val.substring( index + 1, val.length() );
	}
	type = ( type.trim() ).toLowerCase();
	value = value.trim();
    }

    public void setType( String type ) { this.type = type; }
    public String getType() { return type; }

    public void setValue( String value ) { this.value = value; }
    public String getValue() { return value; }

    public String toString() { return type + sep + value; }

    final static String sep = ":";
    String type;
    String value;
}

// ----------------------------------------------------------------

/**
 * Set of attributes
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 */
public class Attributes {
  Vector attributes = new Vector();
  Hashtable ht = new Hashtable();

  public Attributes() {}

  public Attributes( String[] atts ) {
    for ( int i = 0; i < atts.length; i++ ) {
      Attribute att = new Attribute( atts[ i ] );
      attributes.addElement( att );
      ht.put( att.getType(), att.getValue() );
    }
  }

  public String getValue( String type ) {
    return (String)ht.get( type );
  }

  public void add( String type, String value ) {
    Attribute att = new Attribute( type, value );
    attributes.addElement( att );
    ht.put( att.getType(), att.getValue() );
  }

  public void add( String s ) {
    Attribute att = new Attribute( s );
    attributes.addElement( att );
    ht.put( att.getType(), att.getValue() );
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    for ( Enumeration enum = attributes.elements(); enum.hasMoreElements(); ) {
      Attribute att = ( Attribute ) enum.nextElement();
      sb.append( att.toString() ).append( "\n" );
    }
    return sb.toString();
  }
  
}


