// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.http1_0;

import jason.service.http1_0.framework.*;

import java.util.*;
import java.io.*;

/**
 * Factory for document header
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 * @see IntDocFactory
 */
public class DocHttp1Factory implements IntDocFactory {
  Hashtable htMime;
  final String RC = "\n";
  final String DEF_MIME = "text/html";

  public DocHttp1Factory( Hashtable htMime ) {
    this.htMime = htMime;
  }

  // Return the mime type
  private String getMime( String path ) {
    String mime;
    int i = path.lastIndexOf( "." );
    if ( ( mime = (String)htMime.get( path.substring( i + 1, path.length() ) ) ) == null )
      mime = DEF_MIME;
    return mime;
  }

    // Header for HTTP 1.0
  public IntDocHttp buildDoc( String serveur, String path, String code, StringBuffer sbDoc ) {
    String mime = getMime( path );

    StringBuffer sbHeaderHttp = new StringBuffer();
    // Header
    sbHeaderHttp.append( "HTTP/1.0 " + code ).append( RC );
    // Date
    sbHeaderHttp.append( "Date: " + ( new Date() ).toString() ).append( RC );
    // Server name
    sbHeaderHttp.append( "Server: " + serveur ).append( RC );
    // Document size
    sbHeaderHttp.append( "Content-Length: " + sbDoc.length() ).append( RC );
    // Mime
    sbHeaderHttp.append( "Content-Type: " + mime ).append( RC );
    
    sbHeaderHttp.append( RC );

    DocHttp1 doc = new DocHttp1( mime, path, code, sbHeaderHttp, sbDoc );
    return doc;
  }

}


