// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.http1_0;

import jason.core.*;
import jason.core.log.*;
import jason.core.pool.*;

import jason.service.http1_0.cache.*;
import jason.service.http1_0.framework.*;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Main class for HTTP.
 * When a client is connected, the HostHandling object is
 * allocated and receives the Socket connection object.
 *
 * @author Alexandre Brillant <abrillant@wanadoo.fr>
 * @version 1.2
 * @since 1.0
 * @see IntHttpCode
 * @see HostHandling
 * @see SocketService */
public class HTTPService extends jason.core.HTTPService implements IntHttpCode {

    public static String TYPE = "http/1.0";

    public HTTPService() {
	super();
	try {
	    setType( new ServiceType( ServiceType.JASON_SERVICE, new String[] { TYPE } ) );
	} catch( ServiceException exc ) {
	    fireCriticalErrorDetected( exc );
	}
    }

    public void start() {
	try {
	    config = new ComponentFactory( this );
	    config.loadComponents();
	    // Run the thread
	    super.start();
	} catch( ConfigException exc ) {
	    sendErrorMessage( exc.getMessage() );
	    exc.printStackTrace();
	}
    }

    protected Task getTaskForSocket( Socket host ) throws ServiceException {
 	return new HostHandling( this, host, config ); 
    }

    ComponentFactory config;
}
