package test;

import java.io.BufferedWriter;
import jason.core.*;
import java.util.*;

/**
 * Here a straightforward class for a minimal usage. This is an extension for
 * testing the Update capacity and the persistence capacity.
 *
 * @author "Alexandre Brillant" <abrillant@wanadoo.fr>
 * @version 1.0
 * @since 1.0
 * @see AbstractService
 */
public class HelloServiceExt extends AbstractService {

    /**
     * In this constructor, I specify not to use this service in a Thread
     * So it will be run once only. */
    public HelloServiceExt() {
	super();
	setName( "Hello2" );
	setPersistent( true );
	setAutoStop( true );
    }

    /** Date courante */
    private Date cCurrentDate;

    /** Method for persistence demonstration */
    public void setCurrentDate( Date currentDate ) {
	this.cCurrentDate = currentDate;
    }

    /** Method for persistence demonstration */
    public Date getCurrentDate() {
	if ( cCurrentDate == null )
	    setCurrentDate( new Date() );
	return cCurrentDate;
    }

    /**
     * This method will be called by the container */
    public void run() {
	// Show a simple message
	String message = "New message " + getStringProperty( "message" ) + " first at " + getCurrentDate();
	sendInformationMessage( message );
	getServiceContext().cleanWorkingDirectory( true );

	try {
	    // Write it in a temporary file 
	    BufferedWriter mWriter = new BufferedWriter( getServiceContext().getWorkingDirectoryFileWriter( "message.txt" ) );
	    mWriter.write( message );
	    mWriter.close();
	} catch( Exception exc ) {
	    System.err.println( "Can't write to message.txt" );
	}
    }

    public ServiceInfo getInfo() { return new ServiceInfo( "A.Brillant", "06/02", "06/02", "" ); }

}





