package test;

import jason.core.*;
import jason.core.pool.Task;
import jason.core.pool.AbstractTask;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Here a JASon exemple with a service that returns 
 * when a client is connected the current time.
 *
 * @author "Alexandre Brillant" <abrillant@wanadoo.fr>
 * @version 1.0
 * @since 1.0
 * @see SocketService
 */
public class TimeService extends SocketService {

    /**
     * Here we return a task for a client host request. This task will manage the response */
    protected Task getTaskForSocket( Socket host ) throws ServiceException {
	return new TimeTask( host );
    }

    ////////////////

    /**
     * Here the default Task class for a client request */
    private class TimeTask extends AbstractTask {
	private Socket host;

	public TimeTask( Socket host ) {
	    this.host = host;
	}

	public void run() {
	    OutputStream out = null;
	    try {
		// We send the time to the current client
		out = host.getOutputStream();
		BufferedWriter bw = new BufferedWriter( new OutputStreamWriter( out ) );
		bw.write( "" + new Date() );
		bw.newLine();
		bw.flush();
	    } catch( Exception exc ) {
		sendErrorMessage( exc.getMessage() );
	    }
	}
    }
}


