// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason;

import java.io.*;

import jason.core.*;
import jason.core.log.Log;

/**
 * Load each services for a domain name. This services are described in the
 * 'jason.xml' file. This file included references to 'services.xml' file type that
 * contains all needed services. 
 *
 * @author "alexandre brillant" <abrillant@wanadoo.fr>
 * @version 1.1 Support for 'jason.xml'
 * @since 1.0 */
public class ServerLoader { 

    private JasonEnv je;

    public void init( String[] args ) {
	this.args = args;
	try {
	    je = new JasonEnv( new File( "conf/jason.xml" ) );
	    parseArg( args, je );
	    je.init();
	    run();
	} catch( JasonException exc ) {
	    System.err.println( "Can't initialize JASon "  + exc.getMessage() );
	    exc.printStackTrace();
	    System.exit( 1 );
	}
    }

    public void run() {

	je.run();
    }

    // Analysis user command line
    private void parseArg( String[] args, JasonEnv je ) {
	if ( args != null ) {
	    int i = 0;
	    while ( i < args.length ) {
		if ( args[ i ].startsWith( "-" ) ) {
		    String _name = args[ i ].substring( 1, args[ i ].length() );
		    String _value = null;
		    if ( i + 1 < args.length ) 
			_value = args[ i + 1 ];
		    je.setArgumentProperty( _name, _value );
		    if ( _value != null )
			if ( !( _value.startsWith( "-" ) ) )
			    i++;
		}
		i++;
	    }
	}
    }

    private String[] args;

    public static void main( String[] args ) throws Exception {
	ServerLoader server = new ServerLoader();
	server.init( args );
    }

}
