// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import java.io.*;
import java.util.*;

/**
 * Abstract factory for the Server component. Each main element of the AWS server is provided by this class. It
 * hides how the element are found. Singleton mechanism is used for minimized element allocation. 
 * To work correctly, the ComponentFactory need to know how to instantiate each element. The Server Config is a 
 * configuration element that provide information for this factory.
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 */
public abstract class AbstractComponentFactory {

    /**
     * Store the ServerConfig element
     *
     * @param config a <code>ServerConfig</code> value
     * @exception ConfigException if an error occurs
     */
    public AbstractComponentFactory( ServiceConfig config ) throws ConfigException {
	this.config = config;
	if ( config == null ) 
	    throw new ConfigException( "Invalid configuration element" );
	loadComponents();
    }

    // Should be overrided by services
    protected void loadComponents() throws ConfigException {}

    protected final void setComponent( String name, Object component ) {
	map.put( name, component );
    }

    public Object getComponent( String name ) {
	return map.get( name );
    }

    private HashMap map = new HashMap();
    protected ServiceConfig config;
}





