// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

/**
 * Action inside a Service. This kind if object
 * handle a particular client request.
 * Ex : GET, POST, HEAD are actions tied to the 
 * HTTP/1.0 protocol.
 * A action should be a fast processing, there's
 * no possibility to stop it. Service must define
 * a little communication with the param object.
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 */
public interface Action {

    /** @return Name of the action (ex : GET, POST... ) */
    public String getName();
    public void setName( String name );

    /**
     * Initialize the action once 
     * @param config Configuration about the action */
    public void init( ActionConfig config );

    /**
     * Run the action with a particular param */
    public void start( Object param );
}
