// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core;

/**
 * <b>Created Sun Sep 08 21:16:24 2002</b>
 * <p>
 * <code>ServiceContainer</code> builder for static service. This is typically service
 * not described in XML. This service type are normally encoded with its data for working, 
 * however, this is possible to update this value.</code>
 * </p>
 * @author Alexandre Brillant
 * @version 1.0
 */
public class ClassServiceBuilder extends AbstractServiceBuilder {
    private Domain d;
    private ServicesManager sm;

    public ClassServiceBuilder( ServicesManager sm, Domain d ) {
	this.sm = sm;
	this.d = d;
    }

    public ServiceContainer buildService( String className ) throws JasonException {
	super.buildService( className );
	// To correct later for service having a no argument constructor
	// Static singleton ?
	try {
	    Service s = ( Service )Class.forName( className ).newInstance();
	    ServiceContainer _sc = new ServiceContainer( sm, 
							 s,
							 s.getName(), 
							 s.getClass().getName(), 
							 false );
	    _sc.setServiceContainerBuilder( this );
	    return _sc;
	} catch( Exception exc ) {
	    throw new JasonException( "Cannot build service " + className + " : " + exc.getMessage() );
	}
    }

}

// ClassServiceBuilder ends here
