// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

/**
 * <b>Created Wed Mar 27 12:23:35 2002</b>
 * <p>
 * This class is responsable for running the 
 * domain. It means, get all available services and
 * starts it.
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0 */
public class DomainRunner {
    private Domain d;

    public DomainRunner( Domain d ) {
	super();
	this.d = d;
    }

    public void run() {
	ServiceContainerDataModel model = d.getModel();
	d.sendInformationMessage( "Starting the domain '" + d.getName() + "' with " + model.serviceContainerCount() + " services" );
	boolean enabledInfo = "true".equals( System.getProperty( "jason.services.info" ) );

	for ( int i = 0; i < model.serviceContainerCount(); i++ ) {
	    ServiceContainer container = model.getServiceContainerAt( i );
	    Service s = container.getService();
	    if ( s.isEnabled() ) {
		ServiceContext _sc = container.initService();
		d.getServicesManager().notifyInitService( _sc, s );
		if ( !( s instanceof SchedulableService ) ) {
		    d.getServicesManager().notifyStartService( _sc, s );

		    d.sendInformationMessage( "Starting " + 
					      s.getName() + 
					      " " + 
					      s.getVersion() + 
					      " / s" + 
					      _sc.getSkillForHost() + 
					      "." + 
					      _sc.getSkillForRequest() );
		    
		    if ( enabledInfo && ( s.getInfo() != null ) ) {
			d.sendInformationMessage( s.getName() + " : " + s.getInfo() );
		    }

		    s.start();
		} 
	    }
	}

    }
}

// DomainRunner ends here

