// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core;

/**
 * <b>Created Thu May  9 09:52:36 2002</b>
 * <p>
 * A door is a way for integrating easily new services
 * without configuring the jason configuration file. This
 * is tipically a directory that can contains news service
 * loaded at starting. However, user can integrate more
 * service loading type with this interface. It will have
 * to specify a valid class in the <code>jason.xml</code> file.
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 * @see DoorContext
 * @see PathDoor */
public interface Door {

    /**
     * This is call at the beginning of the Door using by the Domain container. The
     * location is normally a value from the jason.xml file. This is a location for
     * service in any format
     * @param context location for services
     * @exception DoorException exception for bad location value */
    public void init( DoorContext context ) throws DoorException;

    /** Run again the door for signaling new service */
    public void update();

    /**
     * @return All available services */
    public Service[] getServices();
    
    /** @return <code>true</code> if this door can be called at runtime for updating a service */
    public boolean isUpdatable();

    /** Write a service. the byte array is the .jas format. 
     @return <code>false</code> if the service can't be written */
    public boolean writeService( String name, byte[] service );

}

// Door ends here

