// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import java.io.*;

/**
 * HTTPService
 * This service is for processable task, it is able to get all the 
 * request from a socket object. 
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0 */
public abstract class HTTPService extends ProcessableService {

    public HTTPService() {
	super();
    }

    /**
     * This method returns an array of bytes with the full user request, this is needed for load
     * balancing between several process. By default this method tries to read until two '\n' or the end of the stream.
     * If your protocol doesn't limit the scope of the request with the same method, please override this method.
     *
     * @param in InputStream from the user request 
     * @return the request if a byte array */
    protected byte[] getHostRequestFromInputStream( InputStream in ) throws IOException {
	ByteArrayOutputStream out = new ByteArrayOutputStream();
	int last1 = 0;
	int last2 = 0;
	int last3 = 0;
	while ( true ) {
	    int n = in.read();
	    if ( n == -1 ) 
		break;
	    // Condition out for HTTP only
	    if ( ( n == 10 ) && 
		 ( last1 == 13 ) && 
		 ( last2 == 10 ) && 
		 ( last3 == 13 ) ) 
		break;

	    out.write( n );

	    last3 = last2;
	    last2 = last1;
	    last1 = n;
	}
	return out.toByteArray();
    }

    /**
     * @return a task for a user socker */
    protected abstract jason.core.pool.Task getTaskForSocket( java.net.Socket host ) throws ServiceException;

}
