// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import jason.core.log.Log;
import java.util.*;

/**
 * Contains information about a runlevel definition :
 * - name,
 * - list of service reference.
 *
 * @author "djefer" <djefer@club-internet.fr>
 * @version 1.0
 * @since 1.0
 */
public class Runlevel {

    /**
     * @param name Name of the runlevel */
    public Runlevel( String name, Service[] servicesRef ) {
	this.name = name;
	services = new Vector();
	this.servicesRef = servicesRef;
    }

    /**
     * @return the name of the runlevel */
    public String getName() { 
	return name;
    }

    /**
     * @return an Enumeration of Runlevel object */
    public Enumeration getServiceReferences() {
	return services.elements();
    }

    /**
     * @param a service reference name */
    void addService( String serviceReference ) {
	for ( int i = 0; i < servicesRef.length; i++ ) {
	    if ( servicesRef[ i ].getName().equals( serviceReference ) ) {
		services.add( servicesRef[ i ] );
		break;
	    }
	}
    }

    /**
     * Running all service from the current runlevel 
     * @param System log */
    void start( Log log ) {
	log.addInfo( "Starting runlevel " + getName() + " :" );
	int service_id = 0;
	for ( Enumeration enum = getServiceReferences(); enum.hasMoreElements(); ) {
	    Service service = (Service)enum.nextElement();
	    log.addInfo( (service_id++) + ". Starting service " + service.getName() + " ... " );
	    service.start();
	}
	log.addInfo( "---------------------------------------------" );
    }

    /**
     * Stopping all service from the current runlevel
     * @param Log log */
    void stop( Log log ) {
	log.addInfo( "Stopping runlevel " + getName() + " :" );
	for ( Enumeration enum = getServiceReferences(); enum.hasMoreElements(); ) {
	    Service service = (Service)enum.nextElement();
	    log.addInfo( "- Stopping " + service.getName() + " ... " );
	    service.stop();
	}
    }

    private Service[] servicesRef;
    private String name;
    private Vector services;
}
