// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core;

import java.util.Vector;

/**
 * <b>Created Fri Sep 06 21:31:49 2002</b>
 * <p>
 * This is a data model for all service container. 
 * This dataModel is available for each Domain.
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 * @see ServiceContainer */
public class ServiceContainerDataModel extends Vector {

    public ServiceContainerDataModel() {
	super();   
    }

    private ServiceContainerDataModelListener cListener = null;

    public void addServiceContainerDataModelListener( ServiceContainerDataModelListener listener ) {
	this.cListener = listener;
    }

    /** @rerturn the service context for the service s */
    public ServiceContext getServiceContextForService( Service s ) {
	ServiceContainer mSc = getServiceContainerForService( s );
	if ( mSc == null )
	    return null;
	return mSc.getServiceContext();
    }

    /** @return the service container tied to the service s. This method doesn't use the equals method
	for detecting the good service container */
    public ServiceContainer getServiceContainerForService( Service s ) {
	for ( int i = 0; i < serviceContainerCount(); i++ ) {
	    ServiceContainer mSc = getServiceContainerAt( i );
	    if ( mSc.getService() == s ) {
		return mSc;
	    }
	}
	return null;
    }

    /** @return a service container tied to a service s matching the name */
    public ServiceContainer getServiceContainerForServiceName( String name ) {
	for ( int i = 0; i < serviceContainerCount(); i++ ) {
	    ServiceContainer mSc = getServiceContainerAt( i );
	    if ( ( mSc.getService() != null ) &&
		 ( mSc.getService().getName().equals( name ) ) ) {
		return mSc;
	    }
	}
	return null;
    }

    /** @return the service container for this index from 0 */
    public ServiceContainer getServiceContainerAt( int i ) {
	return ( ServiceContainer )elementAt( i );
    }

    private boolean cEnabledServiceOverriding = true;

    /** 
	Enabled/Disabled the service overriding. When this property is <code>true</code>, adding
	a service will take into account if an existing service of the same type exists. Then, the old
	service will be remove, in such system the last one has always a priority on the old one */
    public void setEnabledServiceOverriding( boolean ok ) {
	this.cEnabledServiceOverriding = ok;
    }

    public boolean isEnabledServiceOverriding() {
	return cEnabledServiceOverriding;
    }

    /** Store a service container */
    public void addServiceContainer( ServiceContainer sc ) {
	add( sc );
	if ( cListener != null )
	    cListener.notifyAddServiceContainer( sc );
	try {
	    if ( !( sc.getService() instanceof SchedulableService ) )
		if ( isEnabledServiceOverriding() && ( sc.getServiceType() != null ) && ( !sc.getServiceType().isDefault() ) )
		    for ( int i = 0; i < ( serviceContainerCount() - 1 ); i++ ) {
			ServiceContainer _sc = getServiceContainerAt( i );
			if ( sc.getServiceType().equals( _sc.getServiceType() ) ) {
			    // Remove it
			    removeServiceContainer( _sc );
			    break;
			}
		    }
	} catch( Throwable th ) {
	    // Nothing to do
	}
    }

    /** Remove the service container */
    public void removeServiceContainer( ServiceContainer sc ) {
	remove( sc );
	if ( cListener != null )
	    cListener.notifyRemoveServiceContainer( sc );
    }

    /** @return a snapshot for available services */
    public ServiceContainer[] getState() {
	ServiceContainer[] sc = new ServiceContainer[ size() ];
	copyInto( sc );
	return sc;
    }

    /** @return the number of service container */
    public int serviceContainerCount() {
	return size();
    }
}

// ServiceContainerDataModel ends here






