// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import java.util.*;

/**
 * <b>Created Sat Feb 16 19:04:29 2002</b>
 * <p>
 * This class is useful for service data exchange
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0 */
public final class ServiceDataSpace {

    /**
     * Store a data in the Service Data Space */
    public static void setData( String key, Object value ) {
	htData.put( key, value );
	fireSetData( key, value );
    }

    /**
     * Store a data and store it in a vector for the key */
    public static void setDataList( String key, Object value ) {
	Vector v = (Vector)htData.get( key );
	if ( v == null ) {
	    v = new Vector();
	    htData.put( key, v );
	}
    }

    /**
     * @return a data for a key */
    public static Object getData( String key ) {
	return htData.get( key );
    }

    /**
     * @return a vector of data */
    public static Vector getDataList( String key ) {
	return (Vector)htData.get( key );
    }

    /**
     * Return true if the key is already used */
    public static boolean hasData( String key ) {
	return htData.containsKey( key );
    }

    /**
     * Test if the value is inside the tied key list */
    public static boolean hasDataValueForList( String key, 
					       Object value ) {
	if ( !htData.containsKey( key ) ) 
	    return false;
	if ( !( htData.get( key ) instanceof Vector ) ) 
	    return false;
	Vector v = (Vector)htData.get( key );
	return v.contains( value );
    }

    /**
     * Store a listener for a data key 
     * @param key '*' for all data key or the key name 
     * @param listener a listener for a data set */
    public static void addServiceDataListener( String key, 
					       ServiceDataListener listener ) {
	Vector _listeners = ( Vector )listeners.get( key );
	if ( _listeners == null ) {
	    _listeners = new Vector();
	    listeners.put( key, _listeners );
	}
	_listeners.addElement( listener );
    }

    private static void fireSetData( String key, Object value ) {
	if ( listeners.contains( key ) ) {
	    Vector _listener = (Vector)listeners.get( key );
	    fireData( _listener, key, value );
	}
    }

    private static void fireData( Vector _listeners, String key, Object value ) {
	for ( int i = 0; i < _listeners.size(); i++ ) {
	    ( (ServiceDataListener)_listeners.elementAt( i ) ).notifyData( key, value );
	}
    }

    private static void fireGlobalData( String key, Object value ) {
	if ( listeners.contains( "*" ) ) {
	    Vector _listener = (Vector)listeners.get( key );
	    fireData( _listener, key, value );
	}
    }

    private static Hashtable listeners = new Hashtable();
    private static Hashtable htData = new Hashtable();
}

// ServiceDataSpace ends here
