// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * <b>Created Sun Oct  6 20:02:24 2002</b>
 * <p>
 * This is a storable area for service content. This class is able to
 * read all service property by reflection available by a "get" method. This
 * objet is then written by the service container. On the contrary to restore
 * the service state, this objet is able to write the "set" methods.
 * </p>
 * <p>
 * Note that only declared method are taken into account meaning all subclasses are
 * ignored.
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 */
public class ServiceState implements Serializable {
    private Hashtable ht;

    public ServiceState( Service s ) {
	super();   
	ht = new Hashtable();
    }

    /** Get all get service value */
    public void postWrite( Service s ) { 
	if ( s == null )
	    throw new RuntimeException( "Invalid service : null " );
	Class c = s.getClass();
	Method[] mm = c.getDeclaredMethods();
	for ( int i = 0; i < mm.length; i++ ) {
	    Method m = mm[ i ];
	    try {
		if ( m.getName().startsWith( "get" ) ) {
		    Object mObj = m.invoke( s, null );
		    if ( ( mObj instanceof Serializable ) && ( mObj != null ) ) {
			ht.put( m.getName().substring( 3 ), mObj );
		    }
		}
	    } catch( Throwable thw ) {}
	}
    }

    /** Reset the service state */
    public void preRead( Service s ) {
	if ( s == null )
	    throw new RuntimeException( "Invalid service : null " );

	Class c = s.getClass();
	for ( Enumeration enum = ht.keys(); enum.hasMoreElements(); ) {
	    String key = ( "" + enum.nextElement() );
	    Object value = (Object)ht.get( key );
	    Method m;
	    try {
		if ( ( m = c.getMethod( "set" + key, new Class[] { value.getClass() } ) ) != null ) {
		    m.invoke( s, new Object[] { value } ) ;
		}
	    } catch( Throwable thw ) {
		thw.printStackTrace();
	    }
	}
    }

}

// ServiceState ends here
