// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import jason.core.pool.Task;
import jason.core.pool.ThreadCapsule;

/**
 * <b>Created Wed May 15 18:45:45 2002</b>
 * <p>
 * This method will handle user <code>Task</code>. This is
 * an important class for thread load balancing because
 * it is able to use the a thread pooling system with a
 * delegated TaskManager
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 * @see ThreadableService
 * @see AbstractService
 * @see Service
 * @see Task
 */
public class TaskableService extends ThreadableService {

    public TaskableService() {
	super();   

    }

    /**
     * This constructor will let the service be started as a thread automatically
     * with the <code>start</code> method.
     * @param autoStart <code>true</code> for starting the service as a thread automatically */
    public TaskableService( boolean autoStart ) {
	super( autoStart );
    }

    /**
     * This method will dispatch ths task in a new thread or in a queue if no
     * thread are available.
     * @param task a user Task object */
    protected void notifyTask( Task task ) {
	if ( isThreadable() ) {
	    ThreadCapsule _thread = getServiceContext().getThreadCapsule();
	    _thread.setTask( task );
	} else
	    task.runTask();
    }

}

// TaskableService ends here
