// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core;

import org.xml.sax.*;

/**
 * <b>Created Wed Mar 27 10:03:52 2002</b>
 * <p>
 * Default error handler when parsing a configuration file
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 * @see ErrorHandler
 * @see JasonEnv
 */
public class XMLErrorHandler implements ErrorHandler {
    private String file;

    public XMLErrorHandler( String file ) {
	super();   
	this.file = file;
    }

    private boolean errorStatus = false;

    public boolean hasError() { return errorStatus; }

    public void warning( SAXParseException exception ) throws org.xml.sax.SAXException {
	Trace.parsingError( file, exception );
    }

    public void error( SAXParseException exception ) throws org.xml.sax.SAXException {
	errorStatus = true;
	Trace.parsingError( file, exception );	
    }

    public void fatalError( SAXParseException exception ) throws org.xml.sax.SAXException {
	errorStatus = true;
	Trace.parsingError( file, exception );
    }

}

// XMLErrorHandler ends here
