// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core.cache;

/**
 * Here the cache interface for the JASon service. This cache is available in the
 * <code>ServiceContext</code> objet
 * @author Alexandre Brillant
 * @version 1.0
 * @see CacheStrategy
 * @see CacheObject
 */
public interface Cache {
    /**
     * This is a delegate for deciding how to purge the cache */
    public void setCacheStrategy( CacheStrategy strategy );

    /**
     * @return the current cache strategy */
    public CacheStrategy getCacheStrategy();

    /**
     * Store an object in the cache
     * @param key the tied key with the cache object 
     * @param object the final object to restore */
    public void storeObject( Object key, CacheObject object );

    /**
     * Retreive an object from the cache. 
     * @return <code>null</code> if the object is unknown or the good object */
    public CacheObject restoreObject( Object key );

    /**
     * Remove an object of the cache
     * @param key the key object for removing */
    public void removeObject( Object key );

}

