// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core.pool;

import java.net.Socket;

/**
 * <b>Created Fri May 24 15:17:41 2002</b>
 * <p>
 * This class is a task for handling a user Socket. It will
 * automatically close the Socket. 
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 */
public class SocketTask extends AbstractTask {
    protected Socket socket;

    /**
     * @param s user socket, this socket is available by the getSocket method */
    public SocketTask( Socket socket ) {
	super();   
	this.socket = socket;
    }

    /**
     * Run the user code and close the socket */
    public void runTask() {
	super.runTask();
	try { 
	    socket.close();
	} catch( Exception exc ) {}
    }

    /**
     * User must override this method and inserts its handling code */
    protected void run() {}

    /**
     * @return the user socket */
    public Socket getSocket() { return socket; }

}

// SocketTask ends here
