// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core.pool;

import java.util.*;

/**
 * <p>
 * This program is an generic implementation of thread pool engine, which takes the following input
 * a) Size of the pool to be constructed
 * b) Name of the class which implements Runnable (which has a visible default constructor)
 * and constructs a thread pool with active threads that are waiting for activation.
 * once the threads have finished processing they come back and wait once again in the pool.
 * </p>
 * <p>
 * This thread pool engine can be locked i.e. if some internal operation is performed on the pool
 * then it is preferable that the thread engine be locked. Locking ensures that no new threads are issued
 * by the engine. However, the currently executing threads are allowed to continue till they come back
 * to the passivePool.
 * </p>
 * @author Sanjay "Ironluca" Debnath <Ironluca@yahoo.com>
 * @author Alexandre Brillant <abrillant@wanadoo.fr> (Last version)
 * Last Modified - 13/01/02
 */
public class ThreadPoolImpl implements ThreadPool {

    private Vector vThreadCapsule;

    public ThreadPoolImpl( int totalPoolSize ) {
	vThreadCapsule = new Vector();
	initializePools( totalPoolSize );
    }

    protected void initializePools( int totalPoolSize ) {
	for(int i=0;i<totalPoolSize;i++){
	    vThreadCapsule.addElement( new ThreadCapsuleImpl() );
	}
    }

    /** 
     * This method returns a ThreadCapsule object with the client processing thread embedded in it
     * If the method doesn't found a threadcapsulse it return <code>null</<code> */
    public ThreadCapsule getThreadCapsule() {
	for ( int i = 0; i < vThreadCapsule.size(); i++ ) {
	    ThreadCapsule _capsule = (ThreadCapsule)vThreadCapsule.elementAt( i );
	    if ( !_capsule.hasTask() )
		return _capsule;
	}
	return null;
    }
}
