// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.core.service;

import java.util.Hashtable;
import jason.core.AbstractService;

/**
 * <b>Created Thu Jun 13 16:52:55 2002</b>
 * <p>
 * This is a service for debugging only. It traces
 * on the console all the service properties
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 */
public class DebugService extends AbstractService {
    public DebugService() {
	super();
	setName( "debug" );
    }

    // Init the service
    protected void notifyInitReady() {
	java.util.Enumeration enum = getPropertiesName();
	while ( enum.hasMoreElements() ) {
	    String name = (String)enum.nextElement();
	    sendInformationMessage( "name = " + name );
	    sendInformationMessage( "value = " + getValueToString( name ) );
	}
    }

    // Convert a value to a flat string
    private String getValueToString( String name ) {
	Object o = getProperty( name );
	if ( o instanceof Object[] ) {
	    return getValueToStringForArray( (Object[])o );
	} else
	    if ( o instanceof Hashtable ) { 
		return getValueToStringForStruct( (Hashtable)o );
	    } else
		return getValueToStringForObject( o );
    }

    // Basic conversion
    private String getValueToStringForObject( Object o ) {
	return "{type=" + o.getClass() + ", value =" + o + "}";
    }

    // Array case
    public String getValueToStringForArray( Object[] o ) {
	StringBuffer sb = new StringBuffer( "array {" );
	for ( int i = 0; i < o.length; i++ ) {
	    if ( i > 0 )
		sb.append(",");
	    sb.append( o[ i ] );
	}
	sb.append( "}" );
	return sb.toString();
    }

    // Struct case
    public String getValueToStringForStruct( Hashtable o ) {
	return "" + o;
    }

}

// DebugService ends here
