// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core.service;

import java.util.Date;
import java.util.Vector;
import jason.core.*;

public class Scheduler extends InnerService {

    public static ServiceType TYPE = null;
    static {
	try {
	    TYPE = new ServiceType( ServiceType.JASON_SERVICE, "scheduler" );
	} catch( Exception exc ) {}
    }

    /**
     * By default this service is not starting as a thread
     * for limiting cost for no SchedulableService instance */
    public Scheduler() {
	super( false );
	setType( TYPE );
	setName( "scheduler" );
	setThreadLoopDelay( 3000 );
	setVersion( new ServiceVersion( 1, 1 ) );
    }

    /**
     * Manage a schedulable request */
    public ServiceResponse manageRequest( ServiceRequest request ) throws ServiceException {
	if ( request.getQuery() instanceof SchedulableService ) {
	    processSchedulableService( ( SchedulableService )request.getQuery() );
	} else {
	    throw new ServiceException( "Invalid request, waiting for a SchedulableService object query" );
	}
	return ServiceResponse.EMPTY_RESPONSE;
    }

    // Someone has asked a request
    protected void processSchedulableService( SchedulableService s ) {
	sendInformationMessage( "Add schedulable service " + s.getName() );
	resetStartingDataTable( s );
	if ( activeThread == null ) {
	    try {
		enableServiceForThread();
		start();
	    } catch( ServiceException exc ) {}
	}
    }

    protected void run() {
	manageScheduleServices();
    }

    private void manageScheduleServices() {
	Date d = new Date();
	for ( int i = 0; i < vToSchedule.size(); i++ ) {
	    SchedulingData _sd = ( SchedulingData )vToSchedule.elementAt( i );
	    _sd.updatePeriod( d );
	    if ( _sd.isExpired() ) {
		// No service container 
		_sd.getService().start();
	    }
	}
    }

    protected void resetStartingDataTable( SchedulableService s ) {
	if ( !vToSchedule.contains( s ) ) {
	    vToSchedule.addElement( new SchedulingData( s ) );

	    // Reset the thread loop on the lowest service period
	    //	    if ( s.getStartingMinute() > 0 ) {
	    //		setThreadLoopDelay( Math.min( getThreadLoopDelay() / 1000, s.getStartingPeriod() ) * 1000 );
	    ///	    }
	}
    }

    private Vector vToSchedule = new Vector();

    ///////////////////////

    class SchedulingData {
	private boolean periodSupport = false;

	public SchedulingData( SchedulableService s ) {
	    this.s = s;
	    period = ( s.getStartingPeriod() * 60 );
	    periodSupport = ( period > 0 );
	}

	public void updatePeriod( Date d ) {
	    canExpired = false;

	    period -= Math.max( 1, getThreadLoopDelay() / 1000 );

	    if ( periodSupport && ( period <= 0 ) ) {
		canExpired = true;
		period = ( s.getStartingPeriod() * 60 );
	    }

	    if ( !canExpired ) {
		// Check for date
		if ( ( d.getMonth() == s.getStartingMonth() ) || ( s.getStartingMonth() < 0 ) ) {
		    if ( ( d.getDay() == s.getStartingDay() ) || ( s.getStartingDay() < 0 ) ) {
			if ( ( d.getHours() == s.getStartingHour() ) || ( s.getStartingHour() < 0 ) ) {
			    if ( d.getMinutes() == s.getStartingMinute() ) {
				canExpired = true;
			    }
			}
		    }
		}
	    }
	}

	public boolean isExpired() {
	    return canExpired;
	}

	public boolean equals( Object obj ) {
	    if ( obj instanceof SchedulingData ) {
		SchedulingData _scheduler = ( SchedulingData )obj;
		return _scheduler.getService() == getService();
	    } else
		return super.equals( obj );
	}

	public SchedulableService getService() { return s; }

	private double period;
	private boolean canExpired = false;
	private SchedulableService s;
    }

}






