// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.core.service;

import jason.core.Domain;
import jason.core.Door;
import jason.core.DoorDataModel;
import jason.core.JasonException;
import jason.core.ServiceBuilder;
import jason.core.ServiceContainer;
import jason.core.ServiceContextListener;
import jason.core.ServiceType;
import jason.core.SchedulableService;
import jason.core.ServiceContainerDataModel;

/**
 * <b>Created Fri Sep 06 22:09:41 2002</b>
 * <p>
 * This is a service runned regularly, it tries to 
 * detect a service update or a new service from the
 * domain door and the current srvice containers.
 * </p>
 * @author your user name (your user mail)
 * @version 1.0
 * @see SchedulableService
 */
public class UpdatableService extends SchedulableService {

    public static ServiceType TYPE = null;

    static {
	try {
	    TYPE = new ServiceType( ServiceType.JASON_SERVICE, "update" );
	} catch( Exception exc ) {}
    }

    public UpdatableService() {
	super();   
	setStartingPeriod( 0.1 );
	setName( "Updatable" );
	setType( TYPE );
    }

    protected void notifyStartReady() {
	Domain _d = getServiceContext().getDomain();
	ServiceContainerDataModel _scd = _d.getModel();

	// Check for configuration file update

	for ( int i = 0; i < _scd.serviceContainerCount(); i++ ) {
	    ServiceContainer _sc = _scd.getServiceContainerAt( i );

	    if ( _sc.checkForUpdate() ) {
		sendInformationMessage( "Updating the service " + _sc );
		ServiceBuilder mServiceBuilder = _sc.getServiceContainerBuilder();
		if ( mServiceBuilder == null ) {
		    sendErrorMessage( "Invalid service container for the service " + _sc );
		}
		_sc.addServiceContextListener( new ServiceContainerToReplaceListener( _sc ) );
		try {
		    _sc.stop();
		} catch( JasonException exc ) {
		    sendWarningMessage( "Error while stopping " + _sc );
		    sendInnerError( exc );
		}
	    }
	}

	// Check for new service from Door

	DoorDataModel mModel = _d.getDoorModel();
	for ( int i = 0; i < mModel.doorCount(); i++ ) {
	    Door mDoor = ( Door )mModel.getDoorAt( i );
	    mDoor.update();
	}
    }

    /////////////////

    /** This is an inner class for knowing exactly when a service is fully terminated */
    class ServiceContainerToReplaceListener implements ServiceContextListener {
	private ServiceContainer scOld;

	public ServiceContainerToReplaceListener( ServiceContainer sc ) {
	    this.scOld = sc;
	}

	public void notifyInit() {}
	public void notifyStart() {}
	public void notifyStop() {
	    // We need to run a new ServiceContainer 
	    try {
		ServiceContainer mContainer = scOld.getServiceContainerBuilder().rebuildService();
		Domain mDomain = mContainer.getServicesManager().getDomain();
		mDomain.replaceServiceContainer( scOld, mContainer );
	    } catch( JasonException exc ) {
		sendErrorMessage( "Error while updating the service " + scOld.getName() );
	    }
	}
    }

}

// UpdatableService ends here
