// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.framework.protocol;

import jason.framework.protocol.http.*;

/**
 * Factory for all protocol parser
 * This factory limits the scope of the constructor, 
 * this is useful for reseting a protocol handler without
 * correcting all user classes 
 * @author Alexandre Brillant */
public final class ProtocolFactory {

    /**
     * @return a Parser for HTTP request */
    public static HttpRequestParser getHttpRequestParser() {
	try {
	    return new HttpRequestParserImpl();
	} catch( HttpParsingException exc ) {}
	return null;
    }

    /**
     * @return a Response builder for the HTTP protocol. User must pass in parameter the 
     * <code>HttpRequestParser</code> used for parsing the request */
    public static HttpResponseBuilder getHttpResponseBuilder( HttpRequestParser parser ) {
	return new HttpResponseBuilderImpl( parser );
    }

}
