// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.framework.protocol.http;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * <b>Created Thu Jan  3 16:20:34 2002</b>
 * <p>
 * Test for Http response
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 */
public class HttpResponseTester {

    public HttpResponseTester() {
	super();   
    }
  
    public static void showResult( HttpRequestParser parser ) {
	System.out.println( "Protocol = [" + parser.getProtocol() + "]" );
	System.out.println( "Method = [" + parser.getMethod() + "]" );
	System.out.println( "RequestURI = [" + parser.getRequestURI() + "]" );
	System.out.println();
	System.out.println( "Headers" );
	if ( parser.getHeaderNames() != null )
	    for ( Enumeration enum = parser.getHeaderNames(); enum.hasMoreElements(); ) {
		String field = (String)enum.nextElement();
		String value = parser.getHeader( field );
		System.out.println( "." + field + "=>[" + value + "]" );
	    }

	System.out.println();
	System.out.println( "Parameters" );
	System.out.println();

	if ( parser.getParameterNames() != null )
	    for ( Enumeration enum = parser.getParameterNames(); enum.hasMoreElements(); ) {
		String field = (String)enum.nextElement();
		String value = parser.getParameter( field );
		System.out.println( "[" + field + "] =>[" + value + "]" );
	    }	
    }

    // Test for status code response
    public static void test1( HttpResponseBuilder builder, OutputStream output ) throws Exception {
	builder.setStatusCode( StatusCode.STATUS_200 );
	builder.setServer( "Jason/crusader" );
	builder.setContentType( "text/html" );
	builder.setContent( "<html><h1>Test</h1></html>".getBytes() );
	builder.buildResponse( output );
    }

    public static void main(String args[]) throws Exception {
	HttpRequestParser parser = new HttpRequestParserImpl();
	System.out.println( "Waiting for 8080" );
	ServerSocket socket = new ServerSocket( 8080 );
	Socket _socket = socket.accept();
	System.out.println( "Find connection" );
	parser.parse( _socket.getInputStream() );
	HttpResponseBuilder builder = new HttpResponseBuilderImpl( parser );

	ByteArrayOutputStream output = new ByteArrayOutputStream();
	test1( builder, output );
	test1( builder, _socket.getOutputStream() );

	System.out.println( "SENT :" );
	System.out.println( "[" + new String( output.toByteArray() ) + "]" );
    }
}

// HttpResponseTester ends here
