// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin;

import jason.core.service.admin.AdminService;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Client admin : 
 * Use can send START or STOP command followed with a service name
 * Syntax : Client ServiceIP ServerPort Name Password Command Parameter
 * @author djefer@club-internet.fr
 * @version 1.0
 */
public abstract class AbstractClient {

    private ResourceUploadListener cListener;

    public void setResourceUploadListener( ResourceUploadListener listener ) {
	this.cListener = listener;
    }

    private void fireNotifyUpload( int current, int max ) {
	if ( cListener != null ) 
	    cListener.notifyUpload( current, max );
    }

    PrintStream cOut = System.out;

    public void setResponseOutput( PrintStream ps ) {
	this.cOut = ps;
    }

    public PrintStream getResponseOutput() { 
	return cOut;
    }

    public void sendCommand() throws IOException, Exception {
	int _port = Integer.parseInt( port );
	Socket sck = new Socket( host, _port );
	System.out.println( "Communication established..." );
	OutputStream out = sck.getOutputStream();
	InputStream in = sck.getInputStream();
	BufferedWriter bw = new BufferedWriter( new OutputStreamWriter( out ) );
	StringBuffer response = new StringBuffer();
	response.append( user ).append( " " ).append( password ).append( " " ).append( command ).append( " " );

	notifyClientInit();

	File mFileToSend = null;

	if ( "PUT".equals( command ) ) {
	    // Load the tied file
	    File f = new File( param );
	    if ( !f.exists() ) {
		throw new Exception( "Unknown file " + f );
	    }

	    if ( !f.toString().endsWith( ".jas" ) ) {
		throw new Exception( "Invalid file, waiting for a .jas" );
	    }

	    // Extract a valid file name for the server's side
	    String sf = f.toString();
	    int i = 0;
	    if ( ( i = sf.lastIndexOf( "/" ) ) > 0 ) {
		sf = sf.substring( i + 1 );
	    } else
		if ( ( i = sf.lastIndexOf( "\\" ) ) > 0 ) {
		    sf = sf.substring( i + 1 );
		}

	    response.append( sf );
	    response.append( "=" );
	    response.append( f.length() );

	    mFileToSend = f;

	} else
	    response.append( param );

	// Send the command
	bw.write( response.toString() );
	bw.newLine();
	bw.flush();

	// Send a file ?
	if ( mFileToSend != null ) {
	    FileInputStream mInput = new FileInputStream( mFileToSend );
	    
	    int bufferSize = 1024;
	    int max = ( int )( mFileToSend.length() / ( long )bufferSize );

	    try {
		byte[] buffer = new byte[ bufferSize ];
		int c = 0;
		int current = 0;

		while( ( c = ( mInput.read( buffer ) ) ) > 0 ) {
		    out.write( buffer, 0, c );
		    fireNotifyUpload( current++, max );
		    out.flush();
		}

	    } finally {
		fireNotifyUpload( max, max );
		mInput.close();
	    }
	}

	try {

	    // Get the response
	    BufferedReader br = new BufferedReader( new InputStreamReader( in ) );
	    String _rep;
	    while( ( _rep = br.readLine() ) != null ) {
		cOut.println( _rep );
		notifyClientResponse( _rep );
	    }

	    sck.close();

	} catch( Throwable thw ) {
	    System.err.println( thw.getMessage() );
	}

	notifyClientEnd();

	System.out.println();
	System.out.println( "done" );
    }

    protected String host;
    protected String port;
    protected String user;
    protected String password;
    protected String command;
    protected String param;

    public void setHost( String host ) { 
	this.host = host;
    }

    public void setPort( String port ) {
	this.port = port;
    }

    public void setUser( String user ) {
	this.user = user;
    }

    public void setPassword( String password ) {
	this.password = password;
    }

    public void setCommand( String command ) {
	this.command = command;
    }

    public String getCommand() {
	return this.command;
    }

    public void setParam( String param ) {
	this.param = param;
    }

    Vector vListener = null;

    /** Add a listener for the admin request */
    public void addClientResponseListener( ClientResponseListener listener ) {
	if ( vListener == null )
	    vListener = new Vector();
	vListener.add( listener );
    }

    protected void notifyClientInit() {
	if ( vListener == null )
	    return;
	for ( int i = 0; i < vListener.size(); i++ ) {
	    ClientResponseListener mListener = ( ClientResponseListener )vListener.elementAt( i );
	    mListener.notifyInit();
	}
    }

    protected void notifyClientEnd() {
	if ( vListener == null )
	    return;
	for ( int i = 0; i < vListener.size(); i++ ) {
	    ClientResponseListener mListener = ( ClientResponseListener )vListener.elementAt( i );
	    mListener.notifyEnd();
	}
    }

    protected void notifyClientResponse( String response ) {
	if ( vListener == null )
	    return;
	for ( int i = 0; i < vListener.size(); i++ ) {
	    ClientResponseListener mListener = ( ClientResponseListener )vListener.elementAt( i );
	    mListener.notifyResponse( response );
	}
    }


}

