// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin;

import jason.core.service.admin.AdminService;
import java.io.*;
import java.net.*;

/**
 * Client
 * Client admin : 
 * Use can send START or STOP command followed with a service name
 * Syntax : Client ServiceIP ServerPort Name Password Command Parameter
 * @author djefer@club-internet.fr
 * @version 1.0
 */
public class Client extends AbstractClient {

    public Client( String[] args ) throws IOException, Exception {
	super();   
	processArgs( args );
	sendCommand();
    }

    private ClientListener cListener;

    /** This is a single client notified for each <code>sendCommand</code> */
    public void setClientListener( ClientListener listener ) {
	this.cListener = listener;
    }

    public void sendCommand() throws IOException, Exception {
	if ( cListener != null )
	    cListener.notifyAction( getCommand() );
	super.sendCommand();
    }

    public Client() { super(); }

    private void processArgs( String[] args ) throws Exception {
	if ( args.length < 6 )
	    throw new Exception( "Invalid command" );
	setHost( args[ 0 ] );
	setPort( args[ 1 ] );
	setUser( args[ 2 ] );
	setPassword( args[ 3 ] );
	setParam( args[ 5 ] );
	setCommand( args[ 4 ] );
    }

    private static void usage() {
	System.out.println( "Client [SERVER IP] [SERVER PORT] [USER] [PASSWORD] [COMMAND] [SERVICE NAME]" );
	String[] commands = AdminService.AVAILABLE_COMMANDS;
	String[] helps = AdminService.HELP_COMMANDS;
	System.out.println( "COMMAND :" );
	for ( int i = 0; i < commands.length; i++ ) {
	    System.out.println( "- " + commands[ i ] + ":" + helps[ i ] );
	}
    }

    public static void main( String[] args ) {
	try {
	    Client cl = new Client( args );
	} catch( IOException exc ) {
	    System.err.println( exc.getMessage() );
	} catch( Exception exc ) {
	    System.err.println( "Invalid command" );
	    usage();
	} 
    }

}
