// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin.swing;

import jason.service.admin.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * <b>Created Sun Oct  6 17:21:30 2002</b>
 * <p>
 * Here a list of log information for each service
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 */
public class LogView extends JPanel {
    private JTextArea cTa;
    private SwingClient cClient;

    public LogView( SwingClient client ) {
	super();   
	this.cClient = client;
	initUI();
    }

    private void initUI() {
	setLayout( new BorderLayout() );
	cTa = new JTextArea();
	add( new JScrollPane( cTa ) );
	initToolBar();
    }

    private void initToolBar() {
	JToolBar mTb = new JToolBar();
	mTb.add( new UpdateAction() );
	mTb.add( new ClearAction() );
	add( mTb, BorderLayout.NORTH );
    }

    public void setContent( String content ) {
	cTa.setText( content );
    }

    //////////////////////

    class UpdateAction extends AbstractAction implements ClientResponseListener {
	public UpdateAction() {
	    super();
	    putValue( Action.NAME, "Update" );
	}

	public void actionPerformed( ActionEvent e ) {
	    Client mClient = cClient.getClient();
	    mClient.addClientResponseListener( this );
	    mClient.setCommand( "LOG" );
	    mClient.setParam( "*" );
	    try {
		mClient.sendCommand();
	    } catch( Throwable thw ) {
		setContent( thw.getMessage() );
	    }
	}

	public void notifyInit() {}
	public void notifyEnd() {
	    setContent( cClient.getLastResponse() );
	}
	public void notifyResponse( String response ) { }
    }

    class ClearAction extends AbstractAction {
	public ClearAction() {
	    super();
	    putValue( Action.NAME, "Clear" );
	}
	public void actionPerformed( ActionEvent e ) {
	    setContent( "" );
	}
    }

}

// LogView ends here
