// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin.swing;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

/**
 * <b>Created Wed Sep 25 19:09:23 2002</b>
 * <p>
 * Dialog for prompting user to insert a valid login and password value
 * </p>
 * @author Alexandre Brillant (abrillant@wanadoo.fr)
 * @version 1.0
 * @see JDialog */
public class LoginPasswordDialog extends JDialog implements ActionListener {
    private JPasswordField cPF;
    private JTextField cTF;
    private JTextField cHost;
    private JTextField cPort;

    public LoginPasswordDialog() {
	super();
	setModal( true );
	setTitle( "Login & Password" );
	initView();
	initSize();
    }

    private void initSize() {
	pack();
	setLocation( 
		    ( ( getToolkit().getScreenSize().width - getWidth() ) / 2 ),
		    ( ( getToolkit().getScreenSize().height - getHeight() ) / 2 ) );
    }

    private void initView() {
	JPanel mPanel = new JPanel();
	cTF = new JTextField( "jason01" );
	cPF = new JPasswordField( "z86" );
	cHost = new JTextField( "127.0.0.1" );
	cPort = new JTextField( "8085" );

	mPanel.setLayout( new BoxLayout( mPanel, BoxLayout.Y_AXIS ) );
	
	mPanel.add( new JLabel( "Host" ) );
	mPanel.add( cHost );
	
	mPanel.add( new JLabel( "Port" ) );
	mPanel.add( cPort );

	mPanel.add( new JLabel( "Login" ) );
	mPanel.add( cTF );
	mPanel.add( new JLabel( "Passord" ) );
	mPanel.add( cPF );

	getContentPane().add( mPanel, BorderLayout.CENTER );

	JPanel mPanelButton = new JPanel();
	JButton mOk = new JButton( "Ok" );
	mOk.setActionCommand( "ok" );
	JButton mCancel = new JButton( "Cancel" );
	mCancel.setActionCommand( "cancel" );
	mPanelButton.add( mOk );
	mPanelButton.add( mCancel );

	mOk.addActionListener( this );
	mCancel.addActionListener( this );

	getContentPane().add( mPanelButton, BorderLayout.SOUTH );
    }

    public String getHost() { return cHost.getText(); }
    public String getPort() { return cPort.getText(); }
    public String getPassword() { return cPF.getText(); }
    public String getLogin() { return cTF.getText(); }

    private String lastPassword = null;
    private String lastLogin = null;

    public void setVisible( boolean visible ) {
	lastPassword = getPassword();
	lastLogin = getLogin();
	super.setVisible( visible );
    }

    public void actionPerformed( ActionEvent e ) {
	String _command = e.getActionCommand();
	if ( "cancel".equals( _command ) ) {
	    cPF.setText( lastPassword );
	    cTF.setText( lastLogin );
	}
	setVisible( false );
    }

}

// LoginPasswordDialog ends here
