// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin.swing;

import jason.service.admin.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import jason.core.PathDoor;
import javax.swing.filechooser.FileFilter;

/**
 * Here an interface for controlling each JASon service
 *
 * @author Alexandre Brillant <abrillant@wanadoo.fr>
 * @version 1.0
 * @since 1.0
 * @see JFrame
 * @see ClientResponseListener */
public class SwingClient extends JFrame implements ClientResponseListener, 
						   TreeSelectionListener, 
						   ResourceUploadListener, 
						   ClientListener {

    private JTree cTree;
    private DefaultTreeModel cTreeModel;
    private JTextArea cTa;
    private JSplitPane cSp;
    private LoginPasswordDialog cDialogPassword;
    private StatusBar cSb;
    private JTabbedPane cTp;
    private LogView cLv;

    public SwingClient() {

	try {
	    UIManager.setLookAndFeel( new com.incors.plaf.kunststoff.KunststoffLookAndFeel() );
	} catch( UnsupportedLookAndFeelException exc ) {
	    System.err.println( "Impossible de charger le lookAndFeel" );
	}

	initActions();
	initUI();
	initMenuBar();
    }

    public static ImageIcon getImage( String imageName ) {
	ClassLoader cl = ClassLoader.getSystemClassLoader();
	return new ImageIcon( cl.getResource( "jason/service/admin/swing/images/" + imageName + ".gif" ) );
    }

    private StopServiceAction cStopAction;
    private StartServiceAction cStartAction;

    // North toolbar actions
    private void initActions() {
	cActions = new Action[] { 
	    new ConnectionAction(), 
	    cStartAction = new StartServiceAction(),
	    cStopAction = new StopServiceAction() };

	cStartAction.setEnabled( false );
	cStopAction.setEnabled( false );
    }

    private Action[] cActions = null;

    private void initUI() {
	cDialogPassword = new LoginPasswordDialog();

	cSp = new JSplitPane();
	cSp.setOneTouchExpandable( true );

	JPanel mPanelNorth = new JPanel();
	mPanelNorth.setLayout( new BorderLayout() );

	JToolBar mToolBar = new JToolBar();
	for ( int i = 0; i < cActions.length; i++ ) {
	    mToolBar.add( cActions[ i ] );
	}

	cTree = new JTree();
	cTree.setCellRenderer( new ServiceTreeCellRenderer() );
	cTree.addTreeSelectionListener( new SwingTreeListener( this ) );
	cTree.addTreeSelectionListener( this );
	cTreeModel = new DefaultTreeModel( new DefaultMutableTreeNode( "JASon" ) );
	cTree.setModel( cTreeModel );

	mPanelNorth.add( mToolBar, BorderLayout.NORTH );
	mPanelNorth.add( new JScrollPane( cTree ), BorderLayout.CENTER );

	cSp.setLeftComponent( mPanelNorth );

	cTp = new JTabbedPane();

	JPanel _panel = new JPanel();
	_panel.setLayout( new BorderLayout() );

	cTp.addTab( "Service", _panel );
	cTp.addTab( "Log", cLv = new LogView( this ) );

	cSp.setRightComponent( cTp );

	getContentPane().add( cSp );
	cSb = new StatusBar();
	getContentPane().add( cSb, BorderLayout.SOUTH );

	setSize( new Dimension( 800, 600 ) );
	setLocation( 0, 0 );
	setTitle( "JASon client" );
    }

    private void fireStatusBarMessage( String message ) {
	cSb.setMessage( message );
    }

    private void initMenuBar() {
	JMenuBar mb = new JMenuBar();
	JMenu mMenu1 = new JMenu( "Server" );

	JMenuItem mMenuItem1 = new JMenuItem( new LoginAction() );
	JMenuItem mMenuItem4 = new JMenuItem( new ConnectionAction() );
	JMenuItem mMenuItem2 = new JMenuItem( new StopAction() );
	JMenuItem mMenuItem3 = new JMenuItem( new ExitAction() );

	mMenu1.add( mMenuItem1 );
	mMenu1.add( mMenuItem2 );
	mMenu1.addSeparator();
	mMenu1.add( mMenuItem3 );

	JMenu mMenu3 = new JMenu( "Service" );
	mMenu3.add( new JMenuItem( new StartServiceAction() ) );
	mMenu3.add( new JMenuItem( new StopServiceAction() ) );
	mMenu3.addSeparator();
	mMenu3.add( new JMenuItem( new AddServiceAction() ) );

	JMenu mMenu2 = new JMenu( "Help" );
	mMenu2.add( new JMenuItem( new VersionAction() ) );

	mb.add( mMenu1 );
	mb.add( mMenu3 );
	mb.add( mMenu2 );

	setJMenuBar( mb );	
    }

    /////////////////////

    ProgressBarDialog cDialog = null;

    public void notifyUpload( int current, int max ) {
	if ( cDialog == null )
	    cDialog = new ProgressBarDialog();
	if ( ( !( cDialog.isVisible() ) && ( current < max ) ) )
	    cDialog.setVisible( true );

	cDialog.setMaxValue( max );
	cDialog.setCurrentValue( current );
	if ( current >= max )
	    cDialog.setVisible( false );
    }

    public JTree getTree() { 
	return cTree; 
    }

    // Update the right part of the split pane 
    public void notifyViewForNode( JComponent view ) { 
	JComponent _component = view;
	if ( !( view instanceof JScrollPane ) )
	    _component = new JScrollPane( view );
	JPanel mPanel = (JPanel)cTp.getComponentAt( 0 );
	if ( mPanel.getComponentCount() > 0 )
	    mPanel.remove( 0 );
	mPanel.add( view );
	mPanel.invalidate();
	mPanel.validate();
	cTp.setSelectedComponent( mPanel );
    }

    private String cCommandActionListener = null;

    public void notifyResponse( String response ) {
	if ( "connection".equals( cCommandActionListener ) ) {
	    forwardResponseForConnection( response );
	}
    }

    public void notifyInit() {
	if ( "connection".equals( cCommandActionListener ) ) {
	    forwardInitForConnection();
	}
    }

    public void notifyEnd() {
	if ( "connection".equals( cCommandActionListener ) ) {
	    cTree.expandRow( 0 );
	}
    }

    public void valueChanged( TreeSelectionEvent e ) {
	TreePath _tp = e.getPath();
	DefaultMutableTreeNode _node = ( DefaultMutableTreeNode )_tp.getLastPathComponent();
	if ( _node instanceof ServiceNode ) {
	    ServiceNode mSn = ( ServiceNode )_node;
	    cStartAction.setEnabled( mSn.isStarted() );
	    cStopAction.setEnabled( mSn.isStopped() );
	}
    }

    private void forwardResponseForConnection( String response ) {
	StringTokenizer st = new StringTokenizer( response, "\t" );
	String name = st.nextToken();
	String state = st.nextToken();
	String version = st.nextToken();
	String info = st.nextToken();
	if ( "null".equals( info ) )
	    info = "No information";

	ServiceNode mNode = new ServiceNode( name );
	try {
	    mNode.setStarted( 2 == Integer.parseInt( state ) );
	} catch( NumberFormatException exc ) {}

	DefaultMutableTreeNode mVersion = new VersionNode( "Version" );
	mVersion.add( new DefaultMutableTreeNode( version ) );
	DefaultMutableTreeNode mInfo = new InfoNode( "Info" );
	mInfo.add( new DefaultMutableTreeNode( info ) );
	mNode.add( mVersion );
	mNode.add( mInfo );
	( ( DefaultMutableTreeNode )cTreeModel.getRoot() ).add( mNode );
    }

    private void forwardInitForConnection() {
	cTreeModel.setRoot( new DefaultMutableTreeNode( "JASon" ) );
    }

    public String getHost() { return cDialogPassword.getHost(); }
    public String getPort() { return cDialogPassword.getPort(); }
    public String getLogin() { return cDialogPassword.getLogin(); }
    public String getPassword() { return cDialogPassword.getPassword(); }

    public void notifyAction( String action ) {
	fireStatusBarMessage( action );
    }

    private ByteArrayOutputStream cResponseBuffer = null;

    public String getLastResponse() { 
	return ( "" + cResponseBuffer );
    }

    public Client getClient() { 
	Client mClient = new Client();
	mClient.setUser( getLogin() );
	mClient.setPassword( getPassword() );
	mClient.setHost( getHost() );
	mClient.setPort( getPort() );
	mClient.setClientListener( this );
	mClient.setResourceUploadListener( this );
	mClient.setResponseOutput( new PrintStream( cResponseBuffer = new ByteArrayOutputStream() ) ); 
	return mClient;
    }

    /////////////////////

    /** Change of login, password and service location */
    class LoginAction extends AbstractAction {
	public LoginAction() {
	    super();
	    putValue( Action.NAME, "Login" );
	}

	public void actionPerformed( ActionEvent e ) {
	    cDialogPassword.setVisible( true );
	}
    }

    /** Add a new service */
    class AddServiceAction extends AbstractAction {
	public AddServiceAction() {
	    super();
	    putValue( Action.NAME, "Add" );
	    putValue( Action.SMALL_ICON, getImage( "addService" ) );
	}

	public void actionPerformed( ActionEvent e ) {
	    JFileChooser chooser = new JFileChooser();
	    int _ret = chooser.showOpenDialog( SwingClient.this );

	    chooser.setFileFilter( 
				  new FileFilter() {
				      public boolean accept( File f ) { 
					  return f.toString().endsWith( PathDoor.JAS_EXT_FILE );
				      }
				      public String getDescription() {
					  return ".jas (JAS Format)";
				      }
				  } );

	    if ( _ret == JFileChooser.APPROVE_OPTION ) {
		File f = chooser.getSelectedFile();
		Client c = getClient();
		c.setCommand( "PUT" );
		c.setParam( f.toString() );
		try { 
		    ThreadableClient mThread = new ThreadableClient( c );
		    mThread.sendCommand();
		} catch( Throwable th ) {
		    JOptionPane.showMessageDialog( SwingClient.this, "Connection Failure", "Error", JOptionPane.ERROR_MESSAGE );
		    th.printStackTrace();
		}
	    }
	}
    }

    /** Stop the server */
    class StopAction extends AbstractAction {
	public StopAction() {
	    super();
	    putValue( Action.NAME, "Stop" );
	}

	public void actionPerformed( ActionEvent e ) {
	    if ( JOptionPane.showConfirmDialog( SwingClient.this, "Please confirm the action" ) == JOptionPane.YES_OPTION ) {
		Client mClient = getClient();
		mClient.addClientResponseListener( SwingClient.this );
		mClient.setCommand( "STOP" );
		mClient.setParam( "*" );

		try {
		    mClient.sendCommand();
		} catch( Throwable th ) {
		    th.printStackTrace();
		}
	    }
	}
    }

    /** Exit Action */
    class ExitAction extends AbstractAction {
	public ExitAction() {
	    super();
	    putValue( Action.NAME, "Exit" );
	}

	public void actionPerformed( ActionEvent e ) {
	    System.exit( 1 );
	}
    }

    /** Start a service */
    public class StartServiceAction extends AbstractAction {

	public StartServiceAction() {
	    super();
	    putValue( Action.NAME, "Start" );
	    putValue( Action.SMALL_ICON, getImage( "startService" ) );
	}
	
	public void actionPerformed( ActionEvent e ) {
	    Client mClient = getClient();
	    mClient.setCommand( "START" );
	    try {
		mClient.setParam( "" + cTree.getSelectionPath().getLastPathComponent() );
		try {
		    mClient.sendCommand();
		} catch( Throwable th ) {
		    th.printStackTrace();
		}
	    } catch( NullPointerException ex ) {}
	}
    }

    /** Stop a service */
    public class StopServiceAction extends AbstractAction {
	public StopServiceAction() {
	    super();
	    putValue( Action.NAME, "Stop" );
	    putValue( Action.SMALL_ICON, getImage( "stopService" ) );
	}

	public void actionPerformed( ActionEvent e ) {
	    Client mClient = getClient();
	    mClient.setCommand( "STOP" );
	    try {
		mClient.setParam( "" + cTree.getSelectionPath().getLastPathComponent() );
		try {
		    mClient.sendCommand();
		} catch( Throwable th ) {
		    th.printStackTrace();
		}
	    } catch( NullPointerException ex ) {}
	}
    }

    /** Version info */
    public class VersionAction extends AbstractAction {
	public VersionAction() {
	    super();
	    putValue( Action.NAME, "Version" );
	}

	public void actionPerformed( ActionEvent e ) {
	    JOptionPane.showMessageDialog( null, "Client 1.0", "Version", JOptionPane.INFORMATION_MESSAGE );
	}
    }

    /** List all services */
    public class ConnectionAction extends AbstractAction {
	public ConnectionAction() {
	    super();
	    putValue( Action.NAME, "Connection" );
	    putValue( Action.SMALL_ICON, getImage( "connection" ) );
	}

	public void actionPerformed( ActionEvent e ) {
	    cCommandActionListener = "connection";
	    Client mClient = getClient();
	    mClient.setCommand( "LISTE" );
	    mClient.setParam( "*" );
	    mClient.addClientResponseListener( SwingClient.this );
	    try {
		mClient.sendCommand();
	    } catch( Throwable th ) {
		JOptionPane.showMessageDialog( SwingClient.this, "Connection Failure", "Error", JOptionPane.ERROR_MESSAGE );
		th.printStackTrace();
	    }
	}
    }

    //////////////////

    class ServiceTreeCellRenderer extends DefaultTreeCellRenderer {
	private ImageIcon startedIcon = getImage( "serviceStarted" );
	private ImageIcon stoppedIcon = getImage( "serviceStopped" );

	public ServiceTreeCellRenderer() {
	    super();
	}

	public Component getTreeCellRendererComponent(JTree tree,
						      Object value,
						      boolean sel,
						      boolean expanded,
						      boolean leaf,
						      int row,
						      boolean hasFocus) {
	    if ( value instanceof ServiceNode ) {
		ServiceNode mNode = ( ServiceNode )value;
		if ( mNode.isStarted() ) {
		    setIcon( startedIcon );
		} else
		      setIcon( startedIcon );
		setText( "" + value );
		return this;
	    } else
		  return super.getTreeCellRendererComponent( tree, value, sel, expanded, leaf, row, hasFocus );
	}

    }

    public static void main( String[] args ) {
	SwingClient mClient = new SwingClient();
	mClient.setVisible( true );
    }
}
