// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


package jason.service.admin.swing;

import jason.service.admin.Client;
import jason.service.admin.ClientListener;
import jason.service.admin.ClientResponseListener;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import java.awt.Color;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import java.util.StringTokenizer;

/**
 * <b>Created Wed Sep 25 18:12:36 2002</b>
 * <p>
 * Update the right part of the interface depending on the selected node
 * </p>
 * @author "Alexandre Brillant" <abrillant@wanadoo.fr>
 * @version 1.0 */
public class SwingTreeListener implements TreeSelectionListener {
    private SwingClient cView;
    private Client cClient;

    public SwingTreeListener( SwingClient view ) {
	super();   
	this.cView = view;
    }

    public void valueChanged( TreeSelectionEvent e ) {
	TreePath _tp = e.getPath();
	DefaultMutableTreeNode _node = (DefaultMutableTreeNode)_tp.getLastPathComponent();
	cView.notifyViewForNode( buildViewForNode( _node ) );
    }

    ////////////////////////////////

    private JComponent buildViewForNode( DefaultMutableTreeNode node ) {
	if ( node instanceof VersionNode ) 
	    return getViewForVersionNode( ( VersionNode )node );
	else 
	    if ( node instanceof InfoNode ) {
		return getViewForInfoNode( ( InfoNode )node );
	    } else
		if ( node instanceof ServiceNode )
		    return getViewForServiceNode( ( ServiceNode )node );
		else 
		    return new JPanel();
    }

    private JComponent getViewForVersionNode( VersionNode node ) {
	return new JTextArea( "" + node.getUserObject() );
    }

    private JComponent getViewForInfoNode( InfoNode node ) {
	return new JTextArea( "" + node.getUserObject() );
    }

    private JComponent getViewForException( Throwable thw ) {
	ByteArrayOutputStream mOut = new ByteArrayOutputStream();
	thw.printStackTrace( new PrintStream( mOut ) );
	JTextArea _ta = new JTextArea( mOut.toString() );
	_ta.setForeground( Color.red );
	return _ta;
    }

    private JComponent getViewForServiceNode( ServiceNode node ) {
	cClient = new Client();

	cClient.setUser( "jason01" );
	cClient.setPassword( "z86" );
	cClient.setHost( "127.0.0.1" );
	cClient.setPort( "8085" );
	cClient.setCommand( "PROPERTIES" );
	cClient.setParam( "" + node.getUserObject() );

	cClient.setClientListener( (ClientListener)cView );

	PropertyTableBuilder _builder = new PropertyTableBuilder();
	cClient.addClientResponseListener( _builder );

	try {
	    cClient.sendCommand();
	    return _builder;
	} catch( Throwable thw ) {
	    return getViewForException( thw );
	}
    }

    ////////////////////////

    class PropertyTableBuilder extends JTable implements ClientResponseListener {

	private DefaultTableModel cModel;

	public PropertyTableBuilder() {
	    cModel = new DefaultTableModel( new String[] { "Property", "Value" }, 0 );
	    setModel( cModel );
	}

	public void notifyInit() {}
	public void notifyEnd() {}

	public void notifyResponse( String response ) {
	    StringTokenizer _st = new StringTokenizer( response, "=" );
	    String _key = _st.nextToken();
	    String _value = "";
	    if ( _st.hasMoreTokens() )
		 _value = _st.nextToken();
	    cModel.addRow( new Object[] { _key, _value } );
	}

    }
}

// SwingTreeListener ends here

