// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.service.pool;

import java.util.*;

/**This class is the active pool for threads that are right now servicing the clients. This class is implemented
over a java.util.Vector and is mostly a data oriented class.<br/>
*
*Author - Sanjay"Ironluca"Debnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 10/03/2001
*/

class ActivePool extends Vector implements ActivePoolInt{

/**the class for which the activepool is being maintained.*/
        protected Class threadClass;

/**The size of the active pool at any point of time.*/
        protected int presentSize;

/**The only visible constructor of the class.
@param threadClass - The class for which the pool is being maintained.*/
        public ActivePool(Class threadClass){

                this.threadClass=threadClass;
                presentSize=0;

        }//constructor closing


/**This method from Vector class is being overridden to provide extra functionality when ThreadCapsule
objects are being swapped in and out of the ActivePool.*/
        public boolean add(Object threadCapsuleObject){

                if(threadCapsuleObject instanceof jason.service.pool.ThreadCapsuleInt){

                        if(super.add(threadCapsuleObject)){

                                presentSize++;
                                return true;

                        }//if closing
                        else return false;

                }//if closing
                else return false;

        }//add closing


/**This method is overridden to decrement the presentSize of the active pool*/
        public boolean remove(Object threadCapsuleObject){

                if(threadCapsuleObject instanceof jason.service.pool.ThreadCapsule){

                        if(super.remove(threadCapsuleObject)){

                                presentSize--;
                                return true;

                        }//if closing
                        else return false;

                }//if closing
                else return false;

        }//remove closing

/**This method returns the current size of the active pool. The current size cannot be set by any
means, since it may lead to instability of the pool.
@return int The present size of the pool.*/
        public int getPresentSize(){

                return presentSize;

        }//getPresentSize closing

/**This method returns the class for which the ActivePool is being maintained.*/
        public Class getThreadClass(){

                return threadClass;

        }//getThreadClass closing

}
