// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



package jason.service.pool;

import java.util.*;

/**This class represents the passive pool of threads which are waiting to service clients.
*This class is implemented over a java.util.Stack and is more of a data oriented class.<br/>
*
*Author - Sanjay"Ironluca"Debnath <a href="mailto:Ironluca@yahoo.com">Ironluca@yahoo.com</a><br/>
*Last Modified - 9/03/2001
*/

public class PassivePool extends Stack implements PassivePoolInt{

/**Present size of the PassivePool.*/
        protected int presentSize;

/**The class for which the Passive Pool is being maintained.*/
        protected Class threadClass;

/**The only visible constructor for the class. It initializes some internal state.
@param threadClass The Class for which the passive pool is maintained.*/
        public PassivePool(Class threadClass){

                super();
                this.threadClass=threadClass;
                presentSize=0;

        }//constructor closing

/**Returns the presentSize of the PassivePool.*/
        public int getPresentSize(){

                return presentSize;

        }//getPresentSize closing


/**This method is overridden to decrement the presentSize when an object is popped.*/
        public Object pop(){

                if(presentSize<=0) return null;
                else{

                        presentSize--;
                        return super.pop();

                }//else closing

        }//pop closing


/**This method is overridden to increment the presentSize when an object is pushed.*/
        public Object push(Object threadCapsuleObj){

                super.push(threadCapsuleObj);
                presentSize++;
                return threadCapsuleObj;

        }//push closing

/**This method returns the class for which the pool is being maintained.
@return java.lang.Class - The class for which the pool is being maintained.*/
        public Class getThreadClass(){

                return threadClass;

        }//getClass closing


/**This method of Stack class is being overridden - the functionality remains the same as in Vector this 
method reduces the size of the passivePool.*/
        public void setSize(int newSize){

                super.setSize(newSize);

        }//setSize closing

}
