// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



/*This class is an implementation of a generic object pool.It is to be used in cases when a 
*repository of objects are to be maintained. One instance of the class can maintain a pool
*of objects from one particular class. Different instances of the class are to be used when 
*objects of different classes are to be pooled. This pool always returns an object regardless 
*wether objects are present in the pool or not. If at any instant of time no objects are found 
*in the pool - new objects of the class are instantiated and served. However, it only pools
*objects to the capacity as indicated by the user. The excess objects (if created) are deleted
*upon release. Class to be pooled should have a visible defauly constructor.
*
*One interesting feature of this pool is that if ever any object which is taken from this pool
*is dereferenced - then, at the time of the Garbage Collection it shall return back to it's
*parent stack.
*
*The mechanism is simple. A new class is machine generated (ClassGenerator does this
*job) - this special class extends the one to be pooled & also implements an interface
*(StoredObject). The objects in the pool are actually objects of the generated class. One
*restriction that is imposed is that the class to be pooled cannot be a "final" class & must
*have a "visible default constructor". The template for the generated class is in 
*the generatedTempate.tplt file. It must be on the users current directory.
*
*Author - Sanjay"Ironluca"Debnath<Ironluca@yahoo.com>
*Last Modified - 22/05/2001
*
*/

package jason.service.pool.objectpool;


import java.util.*;
import java.io.*;

public class GenericObjectPool extends Stack{

        protected int poolSize;//The initial size of the pool to be constructed.
        protected String className;//the fully qualified class name of the class for which the pool
                                             //is being maintained.
        protected ClassGenerator classGenerator;//This is the reference to the ClassGenerator object
                                                                     //responsible for generating the .class


        public GenericObjectPool(){

                this(50);

        }//empty default constructor closing


        public GenericObjectPool(int  poolSize){

                this.poolSize=poolSize;
                classGenerator=new ClassGenerator();

        }//parameterised constructior closing

/*
*This method is responsible for creating the object pool.
*/
        public void createPool(String className){

                //Get the Class object from the ClassGenerator
                Class objectClass=classGenerator.generateClass(className);
                if(objectClass==null){System.out.println("Could not proceed due to internal error");return;}

                //Create and initialize the pool - the Stack
                for(int i=0;i<poolSize;i++){

                        try{

                                StoredObject storedObject=(StoredObject)(objectClass.newInstance());
                                storedObject.setParent(this);

                        }catch(InstantiationException ie){System.out.println("Cannot proceed - Illegal access");return;}
                         catch(IllegalAccessException iae){System.out.println("Cannot proceed - object cannot be created");return;}
                         catch(ClassCastException cce){System.out.println("Class Cast Exception ");cce.printStackTrace();return;}


                }//for closing

        }//createPool closing

public static void main(String args[]) throws Exception{

GenericObjectPool pool=new GenericObjectPool();
pool.createPool(args[0]);
try{
while(true){

pool.pop();

}//while closing

}catch(Exception e){}

System.gc();
System.out.println("sleeping");Thread.sleep(5000);
try{
while(true){pool.pop();System.out.println("Popped");}
}catch(Exception e){}

System.gc();

}//main closing

}
