// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



/* This class is a singleton data class. The primary objective of this class is to extract data from the 
*protocolConfig.xml file and store it in a data structure for future use of different protocol 
*recognizer classes. At one point only one instance of this class exists. If one instance have already
*been instantiated - the reference to the same is returned. It does not have a visible constructor.
*All interactions for creation of the one instance is done through factory methods.
*This class uses Apache - Xerces XML parser (SAX) to parse the protocolConfig document by default
*
*Author - Sanjay"Ironluca"Debnath<Ironluca@yahoo.com>
*Last Modified - 31/03/2001
*/

package jason.service.requestparser.datahandler;

import java.io.*;
import java.util.*;
import org.apache.xerces.parsers.*;//for SAX Parser class
import org.xml.sax.*;//for ContentHandler interface

public class ProtocolConfigExtractor extends ContentHandlerAdapter implements ProtocolConfigExtractorInt{

protected File configXmlFileName;//This variable stores the reference to the XML protocolConfig file

protected static ProtocolConfigExtractor singletonObject=null;//This variable stores the reference to the single object of
                                                                                            //of the class that is instantiated and shall be used by all parties.

protected static boolean instantiated=false;//This boolean value indicates weather an object of this class
                                                              //has been instantiated or not. If not - then an object is instantiated
                                                             //Otherwise the object is not instantiated.

protected static XMLReader saxParser=null;//This variable stores thereference to the SAX parser used to parse the protocolConfig
                                                                 //document. If it is required, the same parser shall parse a different protocolConfig

protected ProtocolInformationUnitEngineInt [] protocolInformationUnitSensitive;//Stores the ProtocolInformationUnits
                                                                    //for different case Sensitive protocols as inidicated in protocolConfig

protected ProtocolInformationUnitEngineInt [] protocolInformationUnitInsensitive;//Stores the ProtocolInformationUnits
                                                                    //for different case Inensitive protocols as inidicated in protocolConfig

protected Vector tempPorotocolInfoUnit;//Temporary storage to keep ProtocolInformationUnits
                                                             //during initialization

protected Vector tempPatternStringInfo;//Temporary storage to store patternStrings during initialization
protected ProtocolInformationUnitEngineInt tempInfoUnit;//temporary storage for an ProtocolInformationUnit
protected String elementUnderParse;
protected StringBuffer tempStringBuffer;


//This constructor is protected, therefore it is not accessable to any part of the system.
//The main functions of the constructor is to instantiate an object of the class, set the "instantiated"
//variable to "true" and to store the reference to the newly created object for future use.
protected ProtocolConfigExtractor(File configXmlFileName) throws SAXException, IOException{

if(saxParser==null) saxParser=new SAXParser();
saxParser.setContentHandler(this);

tempPorotocolInfoUnit=new Vector();
tempPatternStringInfo=new Vector();
tempStringBuffer=new StringBuffer();

saxParser.parse("file://"+configXmlFileName.getPath());

}//constructor closing




//This method is the factory method which is always used to get a reference to a ProtocolConfigExtractor
//object. this method is called by the user to procure a well initialized object of the class.
public static ProtocolConfigExtractorInt getProtocolConfigExtractorInt(String configXmlFileName) throws FileNotFoundException, SAXException, IOException{

if(instantiated && singletonObject.configXmlFileName.getPath().equals(configXmlFileName))return singletonObject;
else {

singletonObject=new ProtocolConfigExtractor(new File(configXmlFileName));
instantiated=true;
return singletonObject;

}//else closing

}//getProtocolConfigExtractorInt closing


//This method returns the fully qualified file name of the protocolConfig file
public String getConfigXmlFileName(){

return configXmlFileName.getPath();

}//getConfigXmlFileName closing


//If this method is called - the old ProtocolConfigExtractor object is discarded and a new one is 
//instantiated using the information of the protocolConfig file supplied to it. All the implecations of
//calling this method has to be thought by the user before invoking it. 
public void setConfigXmlFileName (String configXmlFileName) throws FileNotFoundException{

/*
code to change the protocolConfig file
and to reinitialize the ProtocolConfigExtractor object come here
*/

}//setConfigXmlFileName closing




//This method returns the SAX Parser used to parse the protocolConfig file
public static XMLReader getSaxParser(){

return saxParser;

}//getSaxParser closing



//This method is provided so that the user can plugin a different SAX Parser than the default one.
//The default SAX Parser is Apache - Xerces SAX Parser (org.xerces.parsers.SAXParser)
public static void setSaxParser(XMLReader saxParser){

ProtocolConfigExtractor.saxParser=saxParser;

}//setSaxParser closing


//This overriden method form ContentHandler interface is responsible to insert the required
//information in the ProtocolInformationUnit object
public void startElement(String namespaceURI, String localName, String qName, Attributes atts){

/*This portionof the algorithm inserts the Protocol - name, version, provider & caseSensitivity into the ProtocolInformationUnit*/
if(localName.equals("protocol")){

tempInfoUnit=new ProtocolInformationUnit();

tempInfoUnit.setProtocolName(atts.getValue(namespaceURI,"name"));
tempInfoUnit.setProtocolVersion(atts.getValue(namespaceURI,"version"));
tempInfoUnit.setProtocolProvider(atts.getValue(namespaceURI,"provider"));
tempInfoUnit.setCaseSensitive((Boolean.valueOf(atts.getValue(namespaceURI,"caseSensitive"))).booleanValue());

}//if closing

/*This variable is important - it is reffered while the CDATA is being processed from an element*/
if(localName.equals("patternString"))elementUnderParse=localName;

}//startElement closing



public void endElement(String namespaceURI, String localName, String qName){

/*This portion of the algorithm converts the tempPatternStringInfo Vector into an array and inserts it into the ProtocolInformationUnit*/
/*It also clears the tempPatternStringInfo Vector for further use*/
if(localName.equals("protocol")){

tempInfoUnit.setPatternString((String [])(tempPatternStringInfo.toArray()));
tempPatternStringInfo.clear();

tempPorotocolInfoUnit.addElement(tempInfoUnit);

}//if closing

/*This portion of the algorithm takes the patternString out of the tempStringBuffer and inserts it into the tempPatternStringInfo Vector*/
/*It also clears the tempStringBuffer for further use and sets the elementUnderParse variable to "null"*/
if(localName.equals("patternString")){

tempPatternStringInfo.addElement(tempStringBuffer.toString());
tempStringBuffer.delete(0,tempStringBuffer.length());
elementUnderParse=null;

}//if closing


}//endElement closing

}

