// Copyright (c) 2002 Alexandre Brillant
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



/*This class represents the information bundil required to recognise a protocol.
*Different instances of this class shall represent informations regarding different protocols
*
*Author - Sanjay"Ironluca"Debnath<Ironluca@yahoo.com>
*Last Modified - 01/04/2001
*/

package jason.service.requestparser.datahandler;

public class ProtocolInformationUnit implements ProtocolInformationUnitInt, ProtocolInformationUnitEngineInt{

protected String protocolName;//Name of the protocol
protected String protocolVersion;//Version of the protocol as String, it shall contain both major as well as minor version together
protected String protocolProvider;//Contains the fully qualified class name of the provider/parser for this protocol
protected boolean caseSensitive;//Indicates wether a protocol is caseSensitive or not : true - case sensitive
                                               //false - case insensitive

protected String [] patternString;//Contains all the pattern strings for a particular protocol match
protected boolean informationUnitLocked;//This variable is used to lock a InformationUnit if internal
                                                              //initialization is taking place. It is highly recommended that
                                                             //engines lock a InformationUnit while it's internal state is being
                                                            //reinitialized. At this point - the engine should return "null" for
                                                           //any request for this InformationUnit.


//Returns the array containing the pattern strings
public String [] getPatternString(){

return patternString;

}//getPatternString closing


//returns the name of the protocol for which this InformationUnit is being maintained
public String getProtocolName(){

return protocolName;

}//getProtocolName closing


//Returns the version (major & minor together) of the protocol for which this InformationUnit
//is being maintained
public String getProtocolVersion(){

return protocolVersion;

}//getProtocolVersion closing


//Returns the fully qualified class name of the provider/parser for this protocol
public String getProtocolProvider(){

return protocolProvider;

}//getProtocolProvider closing


//Returns weather the protocol is case sensitive or not
public boolean isCaseSensitive(){

return caseSensitive;

}//isCaseSensitive closing


/*
*The methods below shall not appear in the interface implemented by this class.
*They are only for internal use (reinitialization) only for members of the same package
*are public in nature.
*/


//Sets the pattern strings to a new set - for internal use (reinitialization) only
public void setPatternString(String [] patternString){

this.patternString=patternString;

}//setPatternString closing


//Sets the name of the protocol to the new one given
public void setProtocolName(String protocolName){

this.protocolName=protocolName;

}//setProtocolName closing


//Sets the version of the protocol(major as well as minor together)
public void setProtocolVersion(String protocolVersion){

this.protocolVersion=protocolVersion;

}//setProtocolVersion closing


//Sets the name of the protocol provoder/parser 
public void setProtocolProvider(String protocolProvider){

this.protocolProvider=protocolProvider;

}//setProtocolProvider closing


//Sets the case sensitivity of the protocol
public void setCaseSensitive(boolean caseSensitive){

this.caseSensitive=caseSensitive;

}//setCaseSensitive closing


//Returns the state of lock on this InformationUnit
public boolean isInformationUnitLocked(){

return informationUnitLocked;

}//isInformationUnitLocked closing


public void setInformationUnitLocked(boolean lockState){

informationUnitLocked=lockState;

}//getInformationUnitLocked closing


}
